import abc
from typing import TYPE_CHECKING, Optional, ClassVar, List

if TYPE_CHECKING:
    from istari_digital_client.api.client_api import ClientApi


class ClientHaving(abc.ABC):
    _client: Optional["ClientApi"] = None
    __client_fields__: ClassVar[List[str]] = []

    @property
    def client(self) -> Optional["ClientApi"]:
        return self._client

    @client.setter
    def client(self, value: "ClientApi"):
        self._client = value

        for name in self.__client_fields__:
            attr = getattr(self, name, None)

            if isinstance(attr, list):
                for item in attr:
                    if hasattr(item, 'client'):
                        item.client = value
            elif attr is not None and hasattr(attr, 'client'):
                attr.client = value



import abc
from functools import cached_property
from pathlib import Path

import istari_digital_core


class Properties:
    """Class for holding file properties.

    Properties are the information about a file that's not it's actual content
    but the information stored on a filesystem *about* the file.

    Things like the file name, size, mime type, description, etc...
    """

    def __init__(self: "Properties", native: istari_digital_core.Properties) -> None:
        self.native: istari_digital_core.Properties = native

    @cached_property
    def name(self: "Properties") -> str:
        """The objects file name.

        This includes the extension.

        For example:
        > x.name
        "foo.JPG"
        > x.stem
        "foo"
        > x.extension
        "jpg"
        > x.suffix
        ".JPG"
        """
        file_name = str(self.native.file_name)
        if file_name.lower().endswith(f".{self.extension}"):
            return file_name
        return ".".join([file_name, self.extension])

    @cached_property
    def stem(self: "Properties") -> str:
        """The objects file name stem.

        The stem is the file name without the suffix.

        For example:
        > x.stem
        "foo"
        > x.name
        "foo.jpg"
        > x.extension
        "jpg"
        """
        return Path(self.name).stem

    @property
    def size(self: "Properties") -> int:
        """The objects file size"""
        return self.native.size

    @property
    def mime_type(self: "Properties") -> str:
        """The mime type of the objects file (if known)"""
        return str(self.native.mime)

    @cached_property
    def suffix(self) -> str:
        """Returns the file name suffix.

        - preserves original casing
        - includes the "."

        Examples:
        > x.name
        "foo.JPG"
        > x.suffix
        ".JPG"
        > x.extension
        "jpg"
        """
        return Path(self.name).suffix

    @property
    def extension(self: "Properties") -> str:
        """The objects file name extension.
        Will always be lower case
        Does not include the ".".

        For example:
        > x.extension
        "jpg"
        > x.name
        "foo.JPG"
        > x.stem
        "foo"
        """
        return self.native.extension.lower()

    @property
    def description(self: "Properties") -> str | None:
        """The description set in the file properties.

        Will return None if not set.
        """

        return self.native.description or None

    @property
    def version_name(self: "Properties") -> str | None:
        """The version name set in the file properties.

        Will return None if not set
        """

        return self.native.version_name or None

    @property
    def external_identifier(self: "Properties") -> str | None:
        """The external identifier set in the file properties.

        Will return None if not set
        """
        return self.native.external_identifier or None

    @property
    def display_name(self: "Properties") -> str | None:
        """The display name set in the file properties.

        Will return None if not set
        """
        return self.native.display_name or None


class PropertiesHaving(abc.ABC):
    @property
    @abc.abstractmethod
    def properties(self: "PropertiesHaving") -> "Properties": ...

    @property
    def extension(self: "PropertiesHaving") -> str:
        return self.properties.extension

    @property
    def name(self: "PropertiesHaving") -> str:
        return self.properties.name

    @property
    def stem(self: "PropertiesHaving") -> str:
        return self.properties.stem

    @property
    def description(self: "PropertiesHaving") -> str | None:
        return self.properties.description

    @property
    def size(self: "PropertiesHaving") -> int:
        return self.properties.size

    @property
    def mime_type(self: "PropertiesHaving") -> str:
        return self.properties.mime_type

    @property
    def version_name(self: "PropertiesHaving") -> str | None:
        return self.properties.version_name

    @property
    def external_identifier(self: "PropertiesHaving") -> str | None:
        return self.properties.external_identifier

    @property
    def display_name(self: "PropertiesHaving") -> str | None:
        return self.properties.display_name



from typing import Generic, List, Optional, TypeVar, ClassVar, Callable, Iterator, Dict, Any
from typing_extensions import Annotated
from pydantic import ConfigDict, Field, BaseModel, PrivateAttr

T = TypeVar('T', bound=BaseModel)


class Pageable(BaseModel, Generic[T]):
    items: List[T]
    total: Optional[Annotated[int, Field(strict=True, ge=0)]]
    page: Optional[Annotated[int, Field(strict=True, ge=1)]]
    size: Optional[Annotated[int, Field(strict=True, ge=1)]]
    pages: Optional[Annotated[int, Field(strict=True, ge=0)]] = None
    __client_fields__: ClassVar[List[str]] = ["items"]
    _list_method: Optional[Callable[..., "Pageable[T]"]] = PrivateAttr(default=None)
    _list_method_args: Optional[Dict[str, Any]] = PrivateAttr(default=None)
    __properties: ClassVar[List[str]] = ["items", "total", "page", "size", "pages"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def iter_pages(self) -> Iterator["Pageable[T]"]:
        """ Iterate over all pages."""
        if self._list_method is None:
            raise ValueError("No list method defined for pagination")

        if self._list_method_args is None:
            raise ValueError("No list method arguments defined for pagination")

        current_page = self.page or 1
        size = self.size or 10

        # Prepare base arguments for the list method, excluding pagination parameters
        # and private attributes.  This allows _list_method to be called without needing to
        # pass pagination parameters explicitly each time.
        base_args = {
            k: v for k, v in self._list_method_args.items()
            if k not in {"page", "size", "self"} and not k.startswith("__")
        }

        while True:
            page = self._list_method(**base_args, page=current_page, size=size)

            if page.total == 0:
                break

            yield page

            current_page += 1

            if page.pages and current_page > page.pages:
                break

    def iter_items(self) -> Iterator[T]:
        """ Iterate over all items across all pages."""
        for page in self.iter_pages():
            yield from page.items

    # WARNING: This method overrides Pydantic's default `__iter__()` behavior,
    # which normally yields `(field_name, value)` pairs.
    def __iter__(self) -> Iterator[T]: # type: ignore[override]
        """Default iteration is over items, not pages."""
        return self.iter_items()



# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from istari_digital_client.models.function_status import FunctionStatus
from istari_digital_client.models.operating_system import OperatingSystem
from istari_digital_client.models.tool_version import ToolVersion
from typing import Optional, Set
from typing_extensions import Self

class Function(BaseModel):
    """
    Function
    """ # noqa: E501
    id: StrictStr
    created: datetime
    name: StrictStr
    version: StrictStr
    module_name: StrictStr
    module_version: StrictStr
    entrypoint: StrictStr
    function_schema: Optional[StrictStr]
    run_command: StrictStr
    created_by_id: StrictStr
    status: FunctionStatus
    tool_name: Optional[StrictStr] = None
    tool_display_name: Optional[StrictStr] = None
    tool_versions: Optional[List[ToolVersion]] = None
    operating_systems: Optional[List[OperatingSystem]] = None
    __properties: ClassVar[List[str]] = ["id", "created", "name", "version", "module_name", "module_version", "entrypoint", "function_schema", "run_command", "created_by_id", "status", "tool_name", "tool_display_name", "tool_versions", "operating_systems"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Function from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of status
        if self.status:
            _dict['status'] = self.status.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in tool_versions (list)
        _items = []
        if self.tool_versions:
            for _item_tool_versions in self.tool_versions:
                if _item_tool_versions:
                    _items.append(_item_tool_versions.to_dict())
            _dict['tool_versions'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in operating_systems (list)
        _items = []
        if self.operating_systems:
            for _item_operating_systems in self.operating_systems:
                if _item_operating_systems:
                    _items.append(_item_operating_systems.to_dict())
            _dict['operating_systems'] = _items
        # set to None if function_schema (nullable) is None
        # and model_fields_set contains the field
        if self.function_schema is None and "function_schema" in self.model_fields_set:
            _dict['function_schema'] = None

        # set to None if tool_name (nullable) is None
        # and model_fields_set contains the field
        if self.tool_name is None and "tool_name" in self.model_fields_set:
            _dict['tool_name'] = None

        # set to None if tool_display_name (nullable) is None
        # and model_fields_set contains the field
        if self.tool_display_name is None and "tool_display_name" in self.model_fields_set:
            _dict['tool_display_name'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Function from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created": obj.get("created"),
            "name": obj.get("name"),
            "version": obj.get("version"),
            "module_name": obj.get("module_name"),
            "module_version": obj.get("module_version"),
            "entrypoint": obj.get("entrypoint"),
            "function_schema": obj.get("function_schema"),
            "run_command": obj.get("run_command"),
            "created_by_id": obj.get("created_by_id"),
            "status": FunctionStatus.from_dict(obj["status"]) if obj.get("status") is not None else None,
            "tool_name": obj.get("tool_name"),
            "tool_display_name": obj.get("tool_display_name"),
            "tool_versions": [ToolVersion.from_dict(_item) for _item in obj["tool_versions"]] if obj.get("tool_versions") is not None else None,
            "operating_systems": [OperatingSystem.from_dict(_item) for _item in obj["operating_systems"]] if obj.get("operating_systems") is not None else None
        })
        return _obj



import abc
from pydantic import StrictStr
from typing import TYPE_CHECKING, Optional, List

from istari_digital_client.models.access_relationship import AccessRelationship
from istari_digital_client.models.access_subject_type import AccessSubjectType
from istari_digital_client.models.access_relation import AccessRelation
from istari_digital_client.models.access_resource_type import AccessResourceType
from istari_digital_client.models.update_access_relationship import UpdateAccessRelationship

if TYPE_CHECKING:
    from istari_digital_client.api.client_api import ClientApi


class Shareable(abc.ABC):
    def _require_client(self) -> "ClientApi":
        """Internal helper to assert that a client exists."""
        client = getattr(self, "client", None)
        if client is None:
            raise ValueError(f"`client` is not set for instance of {self.__class__.__name__}")
        return client

    @property
    def _resource_id(self) -> StrictStr:
        resource_id = getattr(self, "id", None)

        if resource_id is None:
            raise ValueError("id is not set")

        return resource_id

    @property
    def _resource_type(self) -> AccessResourceType:
        class_name = self.__class__.__name__

        try:
            return AccessResourceType(class_name.lower())
        except ValueError as e:
            raise ValueError(
                f"Invalid resource type for {class_name}. "
                f"Ensure the class name is a valid AccessResourceType."
            ) from e

    def create_access(
        self,
        subject_type: AccessSubjectType,
        subject_id: str,
        relation: AccessRelation,
    ) -> AccessRelationship:
        client = self._require_client()

        access_relationship = AccessRelationship(
            subject_type=subject_type,
            subject_id=subject_id,
            relation=relation,
            resource_type=self._resource_type,
            resource_id=self._resource_id,
        )

        return client.create_access(
            access_relationship=access_relationship,
        )

    def create_access_by_email(
        self,
        subject_type: AccessSubjectType,
        subject_email: StrictStr,
        relation: AccessRelation,
    ) -> AccessRelationship:
        client = self._require_client()

        return client.create_access_by_email_for_other_tenants(
            subject_type=subject_type,
            email=subject_email,
            resource_type=self._resource_type,
            resource_id=self._resource_id,
            access_relationship=relation,
        )

    def update_access(
        self,
        subject_type: AccessSubjectType,
        subject_id: StrictStr,
        relation: AccessRelation,
    ) -> AccessRelationship:
        client = self._require_client()

        update_access_relationship = UpdateAccessRelationship(
            relation=relation,
        )

        return client.update_access(
            subject_type=subject_type,
            subject_id=subject_id,
            resource_type=self._resource_type,
            resource_id=self._resource_id,
            update_access_relationship=update_access_relationship,
        )

    def remove_access(
        self,
        subject_type: AccessSubjectType,
        subject_id: StrictStr,
    ) -> None:
        client = self._require_client()

        client.remove_access(
            subject_type=subject_type,
            subject_id=subject_id,
            resource_type=self._resource_type,
            resource_id=self._resource_id,
        )

    def list_access(self) -> List[AccessRelationship]:
        client = self._require_client()

        return client.list_access(
            resource_type=self._resource_type,
            resource_id=self._resource_id,
        )



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set, TYPE_CHECKING
from typing_extensions import Self

from istari_digital_client.models.client_having import ClientHaving

if TYPE_CHECKING:
    from istari_digital_client.models.file import File
    from istari_digital_client.api.client_api import Resource
    from istari_digital_client.models.file_revision import FileRevision


class Product(BaseModel, ClientHaving):
    """
    Product
    """ # noqa: E501
    revision_id: StrictStr
    file_id: Optional[StrictStr] = None
    resource_type: Optional[StrictStr] = None
    resource_id: Optional[StrictStr] = None
    relationship_identifier: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["revision_id", "file_id", "resource_type", "resource_id", "relationship_identifier"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    @property
    def file(self) -> Optional["File"]:
        if self.file_id is None or self.client is None:
            return None
        return self.client.get_file(self.file_id)

    @property
    def resource(self) -> Optional["Resource"]:
        file = self.file
        if self.resource_type is None or self.resource_id is None or self.client is None:
            return None
        return self.client.get_resource(self.resource_type, self.resource_id)

    @property
    def revision(self) -> Optional["FileRevision"]:
        if self.revision_id is None or self.client is None:
            return None
        return self.client.get_revision(self.revision_id)

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Product from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if file_id (nullable) is None
        # and model_fields_set contains the field
        if self.file_id is None and "file_id" in self.model_fields_set:
            _dict['file_id'] = None

        # set to None if resource_type (nullable) is None
        # and model_fields_set contains the field
        if self.resource_type is None and "resource_type" in self.model_fields_set:
            _dict['resource_type'] = None

        # set to None if resource_id (nullable) is None
        # and model_fields_set contains the field
        if self.resource_id is None and "resource_id" in self.model_fields_set:
            _dict['resource_id'] = None

        # set to None if relationship_identifier (nullable) is None
        # and model_fields_set contains the field
        if self.relationship_identifier is None and "relationship_identifier" in self.model_fields_set:
            _dict['relationship_identifier'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Product from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "revision_id": obj.get("revision_id"),
            "file_id": obj.get("file_id"),
            "resource_type": obj.get("resource_type"),
            "resource_id": obj.get("resource_id"),
            "relationship_identifier": obj.get("relationship_identifier")
        })
        return _obj



# coding: utf-8

# flake8: noqa
"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from istari_digital_client.models.access_relation import AccessRelation
from istari_digital_client.models.access_relationship import AccessRelationship
from istari_digital_client.models.access_resource_type import AccessResourceType
from istari_digital_client.models.access_subject_type import AccessSubjectType
from istari_digital_client.models.agent import Agent
from istari_digital_client.models.agent_display_name import AgentDisplayName
from istari_digital_client.models.agent_information import AgentInformation
from istari_digital_client.models.agent_module_version import AgentModuleVersion
from istari_digital_client.models.agent_modules import AgentModules
from istari_digital_client.models.agent_status import AgentStatus
from istari_digital_client.models.agent_status_name import AgentStatusName
from istari_digital_client.models.archive import Archive
from istari_digital_client.models.archive_status import ArchiveStatus
from istari_digital_client.models.archive_status_name import ArchiveStatusName
from istari_digital_client.models.artifact import Artifact
from istari_digital_client.models.check_result import CheckResult
from istari_digital_client.models.comment import Comment
from istari_digital_client.models.control_tag import ControlTag
from istari_digital_client.models.control_tag_revision import ControlTagRevision
from istari_digital_client.models.control_tag_status import ControlTagStatus
from istari_digital_client.models.control_tagging_object_type import ControlTaggingObjectType
from istari_digital_client.models.control_tagging_status import ControlTaggingStatus
from istari_digital_client.models.deprecation_reason import DeprecationReason
from istari_digital_client.models.dry_run_snapshot import DryRunSnapshot
from istari_digital_client.models.file import File
from istari_digital_client.models.file_archive_status import FileArchiveStatus
from istari_digital_client.models.file_control_tagging import FileControlTagging
from istari_digital_client.models.file_revision import FileRevision
from istari_digital_client.models.file_revision_archive_status import FileRevisionArchiveStatus
from istari_digital_client.models.filter_by import FilterBy
from istari_digital_client.models.function import Function
from istari_digital_client.models.function_auth_provider import FunctionAuthProvider
from istari_digital_client.models.function_auth_provider_update import FunctionAuthProviderUpdate
from istari_digital_client.models.function_auth_secret import FunctionAuthSecret
from istari_digital_client.models.function_auth_type import FunctionAuthType
from istari_digital_client.models.function_schema import FunctionSchema
from istari_digital_client.models.function_status import FunctionStatus
from istari_digital_client.models.http_validation_error import HTTPValidationError
from istari_digital_client.models.healthcheck_report import HealthcheckReport
from istari_digital_client.models.input_schema import InputSchema
from istari_digital_client.models.input_type import InputType
from istari_digital_client.models.job import Job
from istari_digital_client.models.job_status import JobStatus
from istari_digital_client.models.job_status_name import JobStatusName
from istari_digital_client.models.model import Model
from istari_digital_client.models.model_list_item import ModelListItem
from istari_digital_client.models.module import Module
from istari_digital_client.models.module_author import ModuleAuthor
from istari_digital_client.models.module_author_manifest import ModuleAuthorManifest
from istari_digital_client.models.module_type import ModuleType
from istari_digital_client.models.module_version import ModuleVersion
from istari_digital_client.models.module_version_status import ModuleVersionStatus
from istari_digital_client.models.new_agent import NewAgent
from istari_digital_client.models.new_agent_display_name import NewAgentDisplayName
from istari_digital_client.models.new_agent_information import NewAgentInformation
from istari_digital_client.models.new_agent_module_version import NewAgentModuleVersion
from istari_digital_client.models.new_agent_status import NewAgentStatus
from istari_digital_client.models.new_control_tag import NewControlTag
from istari_digital_client.models.new_function_auth_secret import NewFunctionAuthSecret
from istari_digital_client.models.new_function_manifest import NewFunctionManifest
from istari_digital_client.models.new_function_schema_manifest import NewFunctionSchemaManifest
from istari_digital_client.models.new_input_schema_manifest import NewInputSchemaManifest
from istari_digital_client.models.new_job_assigned_agent import NewJobAssignedAgent
from istari_digital_client.models.new_module_author import NewModuleAuthor
from istari_digital_client.models.new_module_manifest import NewModuleManifest
from istari_digital_client.models.new_operating_system import NewOperatingSystem
from istari_digital_client.models.new_output_schema_manifest import NewOutputSchemaManifest
from istari_digital_client.models.new_snapshot import NewSnapshot
from istari_digital_client.models.new_snapshot_tag import NewSnapshotTag
from istari_digital_client.models.new_system import NewSystem
from istari_digital_client.models.new_system_configuration import NewSystemConfiguration
from istari_digital_client.models.new_tool import NewTool
from istari_digital_client.models.new_tool_version import NewToolVersion
from istari_digital_client.models.new_tracked_file import NewTrackedFile
from istari_digital_client.models.no_op_response import NoOpResponse
from istari_digital_client.models.operating_system import OperatingSystem
from istari_digital_client.models.output_schema import OutputSchema
from istari_digital_client.models.output_type import OutputType
from istari_digital_client.models.page_agent import PageAgent
from istari_digital_client.models.page_agent_status import PageAgentStatus
from istari_digital_client.models.page_artifact import PageArtifact
from istari_digital_client.models.page_comment import PageComment
from istari_digital_client.models.page_file import PageFile
from istari_digital_client.models.page_function import PageFunction
from istari_digital_client.models.page_function_auth_provider import PageFunctionAuthProvider
from istari_digital_client.models.page_job import PageJob
from istari_digital_client.models.page_model_list_item import PageModelListItem
from istari_digital_client.models.page_module import PageModule
from istari_digital_client.models.page_module_author import PageModuleAuthor
from istari_digital_client.models.page_module_version import PageModuleVersion
from istari_digital_client.models.page_operating_system import PageOperatingSystem
from istari_digital_client.models.page_personal_access_token import PagePersonalAccessToken
from istari_digital_client.models.page_resource_search_item import PageResourceSearchItem
from istari_digital_client.models.page_snapshot import PageSnapshot
from istari_digital_client.models.page_snapshot_item import PageSnapshotItem
from istari_digital_client.models.page_snapshot_revision_search_item import PageSnapshotRevisionSearchItem
from istari_digital_client.models.page_snapshot_tag import PageSnapshotTag
from istari_digital_client.models.page_system import PageSystem
from istari_digital_client.models.page_system_configuration import PageSystemConfiguration
from istari_digital_client.models.page_tool import PageTool
from istari_digital_client.models.page_tool_version import PageToolVersion
from istari_digital_client.models.page_tracked_file import PageTrackedFile
from istari_digital_client.models.patch_op import PatchOp
from istari_digital_client.models.personal_access_token import PersonalAccessToken
from istari_digital_client.models.product import Product
from istari_digital_client.models.resource_archive_status import ResourceArchiveStatus
from istari_digital_client.models.resource_control_tagging import ResourceControlTagging
from istari_digital_client.models.resource_search_item import ResourceSearchItem
from istari_digital_client.models.resource_type import ResourceType
from istari_digital_client.models.response_create_snapshot import ResponseCreateSnapshot
from istari_digital_client.models.restore import Restore
from istari_digital_client.models.snapshot import Snapshot
from istari_digital_client.models.snapshot_item import SnapshotItem
from istari_digital_client.models.snapshot_revision_search_item import SnapshotRevisionSearchItem
from istari_digital_client.models.snapshot_tag import SnapshotTag
from istari_digital_client.models.source import Source
from istari_digital_client.models.subject_info import SubjectInfo
from istari_digital_client.models.system import System
from istari_digital_client.models.system_baseline import SystemBaseline
from istari_digital_client.models.system_configuration import SystemConfiguration
from istari_digital_client.models.tenant_public_key import TenantPublicKey
from istari_digital_client.models.token import Token
from istari_digital_client.models.token_with_properties import TokenWithProperties
from istari_digital_client.models.tool import Tool
from istari_digital_client.models.tool_version import ToolVersion
from istari_digital_client.models.tracked_file import TrackedFile
from istari_digital_client.models.tracked_file_specifier_type import TrackedFileSpecifierType
from istari_digital_client.models.update_access_relationship import UpdateAccessRelationship
from istari_digital_client.models.update_control_tag import UpdateControlTag
from istari_digital_client.models.update_system import UpdateSystem
from istari_digital_client.models.update_tag import UpdateTag
from istari_digital_client.models.update_tool import UpdateTool
from istari_digital_client.models.upload_as import UploadAs
from istari_digital_client.models.usability_status_name import UsabilityStatusName
from istari_digital_client.models.usability_status_params import UsabilityStatusParams
from istari_digital_client.models.user import User
from istari_digital_client.models.user_control_tagging import UserControlTagging
from istari_digital_client.models.user_model_inputs import UserModelInputs
from istari_digital_client.models.user_state import UserState
from istari_digital_client.models.user_state_option import UserStateOption
from istari_digital_client.models.user_type import UserType
from istari_digital_client.models.validation_error import ValidationError
from istari_digital_client.models.validation_error_loc_inner import ValidationErrorLocInner
from istari_digital_client.models.new_source import NewSource
from istari_digital_client.models.properties import Properties



from __future__ import annotations

import json
import logging
import os
import shutil
import tempfile
from datetime import datetime
from pathlib import Path
from typing import Union, TypeAlias, Optional, Iterable

from pydantic import StrictStr

from .api.client_api import ClientApi
from .api_client import ApiClient
from .configuration import Configuration, ConfigurationError
from .models.artifact import Artifact
from .models.comment import Comment
from .models.control_tag import ControlTag
from .models.control_tagging_object_type import ControlTaggingObjectType
from .models.file import File
from .models.function_auth_secret import FunctionAuthSecret
from .models.function_auth_type import FunctionAuthType
from .models.job import Job
from .models.model import Model
from .models.new_function_auth_secret import NewFunctionAuthSecret
from .models.new_source import NewSource
from .models.patch_op import PatchOp
from .models.resource_control_tagging import ResourceControlTagging
from .models.user import User
from .models.user_control_tagging import UserControlTagging
from .models.token import Token

logger = logging.getLogger("istari-digital-client.client")

PathLike = Union[str, os.PathLike, Path]
JSON: TypeAlias = dict[str, "JSON"] | list["JSON"] | str | int | float | bool | None


class Client(ClientApi):
    """Create a new instance of the Istari client

    Args:
        config (Configuration | None): The configuration for the client

    Returns:
        Client: The Istari client instance
    """

    def __init__(
        self,
        config: Configuration | None = None,
    ) -> None:
        config = config or Configuration()

        if not config.registry_url:
            raise ConfigurationError(
                "Registry URL is not set! It must be specified either via an ISTART_REGISTRY_URL env variable or by "
                "explicitly setting the registry_url attribute in the (optional) config object on client initialization"
            )
        if not config.registry_auth_token:
            raise ConfigurationError(
                "Registry auth token is not set! It must be specified either via an ISTARI_REGISTRY_AUTH_TOKEN env "
                "variable or by explicitly setting the registry_auth_token attribute in the (optional) config object "
                "on client initialization"
            )

        self.configuration: Configuration = config

        self._api_client = ApiClient(config)

        super().__init__(self.configuration, self._api_client)

    def __del__(self):
        if (
            self.configuration.filesystem_cache_enabled
            and self.configuration.filesystem_cache_clean_on_exit
            and self.configuration.filesystem_cache_root.exists()
            and self.configuration.filesystem_cache_root.is_dir()
        ):
            logger.debug("Cleaning up cache contents for client exit")
            for child in self.configuration.filesystem_cache_root.iterdir():
                if child.is_dir():
                    logger.debug("deleting cache directory - %s", child)
                    shutil.rmtree(
                        self.configuration.filesystem_cache_root, ignore_errors=True
                    )
                elif child.is_file() and not child.is_symlink():
                    logger.debug("deleting cache file - %s", child)
                    child.unlink(missing_ok=True)
                else:
                    logger.debug(
                        "not deleting cache item (is neither a directory nor a regular file) -  %s",
                        child,
                    )

    def add_artifact(
        self,
        model_id: str,
        path: PathLike,
        sources: list[NewSource | str] | None = None,
        *,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
        display_name: str | None = None,
    ) -> Artifact:
        """Add an artifact

        Args:
            model_id (str): The model to add the artifact to
            path (PathLike): The path to the artifact
            sources (List[NewSource]): The sources of the artifact
            description (str | None): The description of the artifact
            version_name (str | None): The version name of the artifact
            external_identifier (str | None): The external identifier of the artifact
            display_name (str | None): The display name of the artifact

        Returns:
            Artifact: The added artifact

        """
        file_revision = self.create_revision(
            file_path=path,
            sources=sources,
            display_name=display_name,
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
        )

        return self._create_artifact(
            model_id=model_id,
            file_revision=file_revision,
        )

    def update_artifact(
        self,
        artifact_id: str,
        path: PathLike,
        sources: list[NewSource | str] | None = None,
        *,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
        display_name: str | None = None,
    ) -> Artifact:
        salt = self.get_artifact(artifact_id=artifact_id).revision.content_token.salt

        file_revision = self.update_revision_content(
            file_path=path,
            salt=salt,
            sources=sources,
            display_name=display_name,
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
        )

        return self._update_artifact(
            artifact_id=artifact_id,
            file_revision=file_revision,
        )

    def add_comment(
        self,
        resource_id: str,
        path: PathLike,
        description: str | None = None,
    ) -> Comment:
        """Add a comment to a resource

        Args:
            resource_id (str): The resource to add the comment to
            path (PathLike): The path to the comment
            description (str | None): The description of the comment

        Returns:
            Comment: The added comment

        """
        file_revision = self.create_revision(
            file_path=path,
            sources=None,
            display_name=None,
            description=description,
            version_name=None,
            external_identifier=None,
        )

        return self._create_comment(
            resource_id=resource_id,
            file_revision=file_revision,
        )

    def update_comment(
        self,
        comment_id: str,
        path: PathLike,
        description: str | None = None,
    ) -> Comment:
        """Update a comment

        Args:
            comment_id (str | UUID | Comment): The comment to update
            path (PathLike): The path to the comment
            description (str | None): The description of the comment

        Returns:
            Comment: The updated comment

        """

        salt = self.get_comment(comment_id=comment_id).revision.content_token.salt

        file_revision = self.update_revision_content(
            file_path=path,
            salt=salt,
            sources=None,
            display_name=None,
            description=description,
            version_name=None,
            external_identifier=None,
        )

        return self._update_comment(
            comment_id=comment_id,
            file_revision=file_revision,
        )

    def add_file(
        self,
        path: PathLike,
        sources: list[NewSource | str] | None = None,
        *,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
        display_name: str | None = None,
    ) -> File:
        """Add a file

        Args:
            path (PathLike): The path to the file
            sources (List[NewSource] | None): The sources of the file
            description (str | None): The description of the file
            version_name (str | None): The version name of the file
            external_identifier (str | None): The external identifier of the file
            display_name (str | None): The display name of the file

        Returns:
            File: The added file

        """
        file_revision = self.create_revision(
            file_path=path,
            sources=sources,
            display_name=display_name,
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
        )

        return self._create_file(
            file_revision=file_revision,
        )

    def update_file(
        self,
        file_id: str,
        path: PathLike | str,
        sources: list[NewSource | str] | None = None,
        *,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
        display_name: str | None = None,
    ) -> File:
        """Update a file

        Args:
            file_id (str): The file to update
            path (PathLike): The path to the file
            sources (List[NewSource] | None): The sources of the file
            description (str | None): The description of the file
            version_name (str | None): The version name of the file
            external_identifier (str | None): The external identifier of the file
            display_name (str | None): The display name of the file

        Returns:
            File: The updated file

        """
        salt = self.get_file(file_id=file_id).revision.content_token.salt

        file_revision = self.update_revision_content(
            file_path=path,
            salt=salt,
            sources=sources,
            display_name=display_name,
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
        )

        return self._update_file(
            file_id=file_id,
            file_revision=file_revision,
        )

    def update_file_properties(
        self,
        file: File,
        display_name: str | None = None,
        description: str | None = None,
        external_identifier: str | None = None,
        version_name: str | None = None,
    ) -> File:
        """Update file properties

        Args:
            file (File): The file to update
            display_name (str | None): The display name of the file
            description (str | None): The description of the file

        Returns:
            File: The updated file

        """
        return self.update_revision_properties(
            file_revision=file.revision,
            display_name=display_name,
            description=description,
            external_identifier=external_identifier,
            version_name=version_name,
        )

    def add_job(
        self,
        model_id: str,
        function: str,
        *,
        parameters: JSON | None = None,
        parameters_file: PathLike | None = None,
        tool_name: str | None = None,
        tool_version: str | None = None,
        operating_system: str | None = None,
        assigned_agent_id: str | None = None,
        agent_identifier: str | None = None,
        sources: list[NewSource | str] | None = None,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
        display_name: str | None = None,
        **kwargs,
    ) -> "Job":
        """Add a job

        Args:
            model_id (str): The model to add the job to
            function (str): The function of the job
            parameters (JSON | None): The parameters of the job
            parameters_file (PathLike | None): The path to the parameters file
            tool_name (str | None): The name of the tool
            tool_version (str | None): The version of the tool
            operating_system (str | None): The operating system of the agent
            assigned_agent_id (str | None): The agent id for the agent assigned to execute the job
            agent_identifier (str | None): The agent identifier for the agent that created the job
            sources (List[NewSource] | None): The sources of the job
            description (str | None): The description of the job
            version_name (str | None): The version name of the job
            external_identifier (str | None): The external identifier of the job
            display_name (str | None): The display name of the job

        Returns:
            Job: The added job

        """
        parameters_file_is_temp = False
        if parameters_file and (parameters or kwargs):
            raise ValueError(
                "Can't combine a parameters file with explicit parameters or parameter kwargs"
            )
        if not parameters_file:
            if parameters and kwargs:
                raise ValueError(
                    "Can't combine explicit parameters with parameters kwargs"
                )
            parameters = parameters or kwargs
            parameters_file = Path(
                tempfile.NamedTemporaryFile(
                    prefix="parameters", suffix=".json", delete=False
                ).name
            )
            parameters_file.write_text(json.dumps(parameters, indent=4))
            parameters_file_is_temp = True
        parameters_file = Path(parameters_file)
        try:
            file_revision = self.create_revision(
                file_path=str(parameters_file),
                sources=sources,
                display_name=display_name,
                description=description,
                version_name=version_name,
                external_identifier=external_identifier,
            )

            openapi_job = self._create_model_job(
                model_id=model_id,
                function_name=function,
                file_revision=file_revision,
                tool_name=tool_name,
                tool_version=tool_version,
                operating_system=operating_system,
                assigned_agent_id=assigned_agent_id,
                agent_identifier=agent_identifier,
            )
        finally:
            if parameters_file_is_temp:
                if parameters_file.exists():
                    parameters_file.unlink(missing_ok=True)

        return openapi_job

    def update_job(
        self,
        job_id: str,
        path: PathLike,
        sources: list[NewSource | str] | None = None,
        *,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
        display_name: str | None = None,
    ) -> Job:
        """Update a job

        Args:
            job_id (str): The job to update
            path (PathLike): The path to the job
            sources (List[NewSource] | None): The sources of the job
            description (str | None): The description of the job
            version_name (str | None): The version name of the job
            external_identifier (str | None): The external identifier of the job
            display_name (str | None): The display name of the job

        Returns:
            Job: The updated job

        """
        salt = self.get_job(job_id=job_id).revision.content_token.salt

        file_revision = self.update_revision_content(
            file_path=path,
            salt=salt,
            sources=sources,
            display_name=display_name,
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
        )

        return self._update_job(
            job_id=job_id,
            file_revision=file_revision,
        )

    def add_model(
        self,
        path: PathLike,
        sources: list[NewSource | str] | None = None,
        *,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
        display_name: str | None = None,
    ) -> Model:
        """Add a model

        Args:
            path (PathLike): The path to the model
            sources (List[NewSource] | None): The sources of the model
            description (str | None): The description of the model
            version_name (str | None): The version name of the model
            external_identifier (str | None): The external identifier of the model
            display_name (str | None): The display name of the model

        Returns:
            model: The added model

        """
        file_revision = self.create_revision(
            file_path=path,
            sources=sources,
            display_name=display_name,
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
        )
        return self._create_model(
            file_revision=file_revision,
        )

    def update_model(
        self,
        model_id: str,
        path: PathLike,
        sources: list[NewSource | str] | None = None,
        *,
        description: str | None = None,
        version_name: str | None = None,
        external_identifier: str | None = None,
        display_name: str | None = None,
    ) -> Model:
        """Update a model

        Args:
            model_id (str): The model to update
            path (PathLike): The path to the model
            sources (List[NewSource] | None): The sources of the model
            description (str | None): The description of the model
            version_name (str | None): The version name of the model
            external_identifier (str | None): The external identifier of the model
            display_name (str | None): The display name of the model

        Returns:
            model: The updated model

        """
        salt = self.get_model(model_id=model_id).revision.content_token.salt

        file_revision = self.update_revision_content(
            file_path=path,
            salt=salt,
            sources=sources,
            display_name=display_name,
            description=description,
            version_name=version_name,
            external_identifier=external_identifier,
        )

        return self._update_model(
            model_id=model_id,
            file_revision=file_revision,
        )

    def add_function_auth_secret(
        self,
        auth_provider_name: str,
        function_auth_type: FunctionAuthType,
        path: PathLike,
        expiration: Optional[datetime] = None,
    ) -> FunctionAuthSecret:
        path = Path(path)
        if not path.exists():
            raise FileNotFoundError(path)

        file_revision = self.create_secret_revision(
            file_path=path,
        )

        # Generate content token for the secret
        # This is a different process than the file revision
        # because the secret is encrypted and we need to
        # generate a token for the plain secret content
        with open(path, "rb") as f:
            secret_content = f.read()
            token: Token = Token.from_bytes(secret_content)

        secret = NewFunctionAuthSecret(
            auth_provider_name=auth_provider_name,
            revision=file_revision,
            function_auth_type=function_auth_type,
            expiration=expiration,
            sha=token.sha,
            salt=token.salt,
        )

        return self._create_function_auth_secret(secret)

    def add_user_control_taggings(
        self,
        user_id: StrictStr,
        control_tag_ids: Iterable[StrictStr],
        reason: Optional[StrictStr] = None,
    ) -> list[UserControlTagging]:
        """Assign one or more control tags to a model

         Args:
             user_id (str): The id of the user to assign control tag access.
             control_tag_id Iterable[str]:  The ids of the control tags to assign access to.
             reason (Optional[str]) The reson for the assignment (optionsl).

         Returns:
             list[UserControlTagging]: A list of UserControlTagging objects, one for each tag assignment made.

        The list resource control taggings returned may be more than the number of control tags assigned,
        as when a tag is applied to a model, the tagging is applied to each of its child artifacts as well.

        Note: The calling user must be a customer admin on the tenant the target user is a member of or the operation
        will fail with a permission denied error.

        """

        return self.patch_user_control_taggings(
            user_id, list(control_tag_ids), patch_op=PatchOp.SET, reason=reason
        )

    def remove_user_control_taggings(
        self,
        user_id: StrictStr,
        control_tag_ids: Iterable[StrictStr],
        reason: Optional[StrictStr] = None,
    ) -> list[UserControlTagging]:
        """Remove (archive) one or more control tag access assignments from a user.

         Args:
             user_id (str): The id of the user to remove control tag access from.
             control_tag_id Iterable[str]:  The ids of the control tags to remove access assignments fro
             reason (Optional[str]) The reson for the assignment (optionsl).

         Returns:
             list[UserControlTagging]: A list of UserControlTagging objects, one for each tag assignment made.

        Note: The calling user must be a customer admin on the tenant the target user is a member of or the operation
        will fail with a permission denied error.

        """

        return self.patch_user_control_taggings(
            user_id, list(control_tag_ids), patch_op=PatchOp.DELETE, reason=reason
        )

    def add_model_control_taggings(
        self,
        model_id: StrictStr,
        control_tag_ids: Iterable[StrictStr],
        reason: Optional[StrictStr] = None,
    ) -> list[ResourceControlTagging]:
        """Assign one or more control tags to a model

         Args:
             model_id (str): The model to assign the control tag to.
             control_tag_id Iterable[str]:  The ids of the control tags to assign.
             reason (Optional[str]) The reson for the assignment (optionsl).

         Returns:
             list[ResourceControlTagging]: A list of ResourceControlTagging objects, one for each tag assignment made.

        The list resource control taggings returned may be more than the number of control tags assigned,
        as when a tag is applied to a model, the tagging is applied to each of its child artifacts as well.

        Note: Owner or administrator access to the model is required to modify control tag assignments.

        """

        return self.patch_model_control_taggings(
            model_id, list(control_tag_ids), patch_op=PatchOp.SET, reason=reason
        )

    def remove_model_control_taggings(
        self,
        model_id: StrictStr,
        control_tag_ids: Iterable[StrictStr],
        reason: Optional[StrictStr] = None,
    ) -> list[ResourceControlTagging]:
        """Remove (archive) one or more control tag assignments from a model.

        Args:
            model_id (str): The id of the model to remove the control tag assignment from.
            control_tag_id Iterable[str]:  The ids of the control tags to assign.
            reason (Optional[str]) The reson for the assignment (optionsl).

        Returns:
            list[ResourceControlTagging]: A list of ResourceControlTagging objects,one for each tagging archived.

        Note: Owner or administrator access to the model its parent model is required to modify control tag
        assignments.

        """

        return self.patch_model_control_taggings(
            model_id, list(control_tag_ids), patch_op=PatchOp.DELETE, reason=reason
        )

    def add_artifact_control_taggings(
        self,
        artifact_id: StrictStr,
        control_tag_ids: Iterable[StrictStr],
        reason: Optional[StrictStr] = None,
    ) -> list[ResourceControlTagging]:
        """Assign one or more control tags to an artifact,

        Args:
            artifact_id (str): The id of the artifact to assign the control tag to.
            control_tag_id Iterable[str]:  The ids of the control tags to assign.
            reason (Optional[str]) The reson for the assignment (optionsl).

        Returns:
            list[ResourceControlTagging]: A list of ResourceControlTagging objects, one for each tag assignment made.

        Note: Owner or administrator access to the artifactits parent model is required to modify control tag
        assignments.

        """

        return self.patch_artifact_control_taggings(
            artifact_id, list(control_tag_ids), patch_op=PatchOp.SET, reason=reason
        )

    def remove_artifact_control_taggings(
        self,
        artifact_id: StrictStr,
        control_tag_ids: Iterable[StrictStr],
        reason: Optional[StrictStr] = None,
    ) -> list[ResourceControlTagging]:
        """Archive one or more control tagings on  amodel

        Args:
            artifact_id (str): The id of the artifact to remove the control tag assignment from.
            control_tag_id Iterable[str]:  The ids of the control tags to assign.
            reason (Optional[str]) The reson for the assignment (optionsl).

        Returns:
            list[ResourceControlTagging]: A list of ResourceControlTagging objects, one for each resulting taggging applied.

        Note: Owner or administrator access to the artifactits parent model is required to modify control tag
        assignments.

        """

        return self.patch_artifact_control_taggings(
            artifact_id, list(control_tag_ids), patch_op=PatchOp.DELETE, reason=reason
        )

    def get_model_control_tags(self, model_id: StrictStr) -> list[ControlTag]:
        """Get list of control tags for the active control taggings on a model.
        Args:
            model_id (str): The id of the model to get the assigned control tags for.

        Returns:
             list[ControlTag};  The list of control tags assigned to the model.

        """

        return self.get_object_control_tags(ControlTaggingObjectType.MODEL, model_id)

    def get_artifact_control_tags(self, artifact_id: StrictStr) -> list[ControlTag]:
        """Get list of control tags for the active control taggings on a model.
        Args:
            artifact_id (str): The id of the model to get the assigned control tags for.

        Returns:
             list[ControlTag};  The list of control tags assigned to the model.

        """

        return self.get_object_control_tags(
            ControlTaggingObjectType.ARTIFACT, artifact_id
        )

    def get_user_control_tags(self, user_id: StrictStr) -> list[ControlTag]:
        """Get list of control tags a user has been assigned access to.
        Args:
            user_id (str): The id of the user to get the assigned control tags for.

        Returns:
             list[ControlTag};  The list of control tags the user has been assigned access to.

        """

        return self.get_object_control_tags(ControlTaggingObjectType.USER, user_id)

    def get_user(self, user_id: StrictStr) -> User:
        """Get a user from the registry
        Args:
            user_id (str): The id of the user to get the assigned control tags for.

        Returns:
             list[ControlTag};  The list of control tags the user has been assigned access to.

        This method simply a convenience wrapper for "get_user_by_id" added for
        "get" method naming convention consistency (get_model, get_artifact, etc...)
        """

        return self.get_user_by_id(user_id)




from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Set
from typing_extensions import Self

from istari_digital_client.models.token import Token
from istari_digital_client.models.readable import Readable
from istari_digital_client.models.client_having import ClientHaving

class SnapshotRevisionSearchItem(BaseModel, ClientHaving, Readable):
    """
    SnapshotRevisionSearchItem
    """ # noqa: E501
    file_id: Optional[StrictStr]
    content_token: Token
    properties_token: Token
    name: Optional[StrictStr] = None
    extension: Optional[StrictStr] = None
    size: Optional[StrictInt] = None
    description: Optional[StrictStr] = None
    mime: Optional[StrictStr] = None
    version_name: Optional[StrictStr] = None
    external_identifier: Optional[StrictStr] = None
    display_name: Optional[StrictStr] = None
    schema_version: Optional[StrictStr] = None
    created_by_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["file_id", "content_token", "properties_token", "name", "extension", "size", "description", "mime", "version_name", "external_identifier", "display_name", "schema_version", "created_by_id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def read_bytes(self) -> bytes:
        if self.client is None:
            raise ValueError("Client is not set")

        return self.client.read_contents(self.content_token)


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SnapshotRevisionSearchItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of content_token
        if self.content_token:
            _dict['content_token'] = self.content_token.to_dict()
        # override the default output from pydantic by calling `to_dict()` of properties_token
        if self.properties_token:
            _dict['properties_token'] = self.properties_token.to_dict()
        # set to None if file_id (nullable) is None
        # and model_fields_set contains the field
        if self.file_id is None and "file_id" in self.model_fields_set:
            _dict['file_id'] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['name'] = None

        # set to None if extension (nullable) is None
        # and model_fields_set contains the field
        if self.extension is None and "extension" in self.model_fields_set:
            _dict['extension'] = None

        # set to None if size (nullable) is None
        # and model_fields_set contains the field
        if self.size is None and "size" in self.model_fields_set:
            _dict['size'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if mime (nullable) is None
        # and model_fields_set contains the field
        if self.mime is None and "mime" in self.model_fields_set:
            _dict['mime'] = None

        # set to None if version_name (nullable) is None
        # and model_fields_set contains the field
        if self.version_name is None and "version_name" in self.model_fields_set:
            _dict['version_name'] = None

        # set to None if external_identifier (nullable) is None
        # and model_fields_set contains the field
        if self.external_identifier is None and "external_identifier" in self.model_fields_set:
            _dict['external_identifier'] = None

        # set to None if display_name (nullable) is None
        # and model_fields_set contains the field
        if self.display_name is None and "display_name" in self.model_fields_set:
            _dict['display_name'] = None

        # set to None if schema_version (nullable) is None
        # and model_fields_set contains the field
        if self.schema_version is None and "schema_version" in self.model_fields_set:
            _dict['schema_version'] = None

        # set to None if created_by_id (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_id is None and "created_by_id" in self.model_fields_set:
            _dict['created_by_id'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SnapshotRevisionSearchItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "file_id": obj.get("file_id"),
            "content_token": Token.from_dict(obj["content_token"]) if obj.get("content_token") is not None else None,
            "properties_token": Token.from_dict(obj["properties_token"]) if obj.get("properties_token") is not None else None,
            "name": obj.get("name"),
            "extension": obj.get("extension"),
            "size": obj.get("size"),
            "description": obj.get("description"),
            "mime": obj.get("mime"),
            "version_name": obj.get("version_name"),
            "external_identifier": obj.get("external_identifier"),
            "display_name": obj.get("display_name"),
            "schema_version": obj.get("schema_version"),
            "created_by_id": obj.get("created_by_id")
        })
        return _obj



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List
from istari_digital_client.models.file import File
from typing import Optional, Set, TYPE_CHECKING
from typing_extensions import Self

from istari_digital_client.models.file_having import FileHaving
from istari_digital_client.models.client_having import ClientHaving

if TYPE_CHECKING:
    from istari_digital_client.models.model import Model
    from istari_digital_client.models.page_artifact import PageArtifact
    from istari_digital_client.models.page_comment import PageComment
    from istari_digital_client.models.page_job import PageJob


class ModelListItem(ClientHaving, FileHaving):
    """
    Model class with unresolved sub-class replaced with lists of uuids.
    """ # noqa: E501
    id: StrictStr
    created: datetime
    file: File
    archive_status: StrictStr
    created_by_id: StrictStr
    comment_ids: Optional[List[StrictStr]] = None
    artifact_ids: Optional[List[StrictStr]] = None
    job_ids: Optional[List[StrictStr]] = None
    __client_fields__: ClassVar[List[str]] = ["file"]
    __properties: ClassVar[List[str]] = ["id", "created", "file", "archive_status", "created_by_id", "comment_ids", "artifact_ids", "job_ids"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    @property
    def model(self) -> Optional["Model"]:
        if not self.client:
            return None
        return self.client.get_model(self.id)

    @property
    def comments(self) -> Optional[PageComment]:
        if not self.comment_ids or not self.client:
            return None
        return self.client.list_model_comments(self.id)

    @property
    def artifacts(self) -> Optional[PageArtifact]:
        if not self.artifact_ids or not self.client:
            return None
        return self.client.list_model_artifacts(self.id)

    @property
    def jobs(self) -> Optional[PageJob]:
        if not self.job_ids or not self.client:
            return None
        return self.client.list_model_jobs(self.id)

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ModelListItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of file
        if self.file:
            _dict['file'] = self.file.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ModelListItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created": obj.get("created"),
            "file": File.from_dict(obj["file"]) if obj.get("file") is not None else None,
            "archive_status": obj.get("archive_status"),
            "created_by_id": obj.get("created_by_id"),
            "comment_ids": obj.get("comment_ids"),
            "artifact_ids": obj.get("artifact_ids"),
            "job_ids": obj.get("job_ids")
        })
        return _obj



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Set, TYPE_CHECKING
from typing_extensions import Self

from istari_digital_client.models.system_configuration import SystemConfiguration
from istari_digital_client.models.snapshot import Snapshot
from istari_digital_client.models.snapshot_tag import SnapshotTag
from istari_digital_client.models.shareable import Shareable
from istari_digital_client.models.archivable import Archivable
from istari_digital_client.models.client_having import ClientHaving


class System(BaseModel, ClientHaving, Shareable, Archivable):
    """
    System
    """ # noqa: E501
    id: StrictStr
    created: datetime
    created_by_id: StrictStr
    name: StrictStr
    description: StrictStr
    archive_status: StrictStr
    configurations: Optional[List[SystemConfiguration]] = None
    baseline_tagged_snapshot_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["id", "created", "created_by_id", "name", "description", "archive_status", "configurations", "baseline_tagged_snapshot_id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def get_json_file_contents_by_snapshot(self, snapshot: Optional["Snapshot"] = None) -> Dict[str, Any]:
        """
        Retrieves the contents of all JSON files for a given snapshot.

        If no snapshot ID is provided, the baseline_tagged_snapshot_id will be used.
        Raises an error if neither is available or if the client is not set.

        Args:
            snapshot (Optional[Snapshot]): The snapshot to retrieve JSON files from.

        Returns:
            Dict[str, Any]: Dictionary including metadata and contents of each JSON file.
        """
        if self.client is None:
            raise ValueError("Client is not set. Please set the client before calling this method.")

        if snapshot is None:
            baseline_snapshot_id = self.baseline_tagged_snapshot_id or self.client.get_system_baseline(system_id=self.id).snapshot_id
            if baseline_snapshot_id is None:
                raise ValueError("No snapshot provided and no baseline snapshot exists.")
            snapshot = self.client.get_snapshot(snapshot_id=baseline_snapshot_id)

        result: Dict[str, Any] = {"metadata": {
            "system_name": self.name,
            "system_id": self.id,
            "system_snapshot_id": snapshot.id,
            "system_snapshot_created": snapshot.created.isoformat(),
        }}

        snapshot_revision_items = self.client.list_snapshot_revisions(
            snapshot_id=snapshot.id,
            size=100,
            extension=[".json"]
        )

        for revision_item in snapshot_revision_items:
            if revision_item.name:
                result[revision_item.name] = revision_item.read_json()

        return result

    def get_json_file_contents_by_snapshot_tag(self, snapshot_tag: SnapshotTag) -> Dict[str, Any]:
        """
        Retrieves the contents of all JSON files for a given snapshot tag.

        Args:
            snapshot_tag (SnapshotTag): The snapshot tag to retrieve JSON files from.

        Returns:
            Dict[str, Any]: Dictionary including metadata and contents of each JSON file.
        """
        if self.client is None:
            raise ValueError("Client is not set. Please set the client before calling this method.")

        snapshot = self.client.get_snapshot(snapshot_id=snapshot_tag.snapshot_id)

        return self.get_json_file_contents_by_snapshot(snapshot=snapshot)

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of System from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in configurations (list)
        _items = []
        if self.configurations:
            for _item_configurations in self.configurations:
                if _item_configurations:
                    _items.append(_item_configurations.to_dict())
            _dict['configurations'] = _items
        # set to None if baseline_tagged_snapshot_id (nullable) is None
        # and model_fields_set contains the field
        if self.baseline_tagged_snapshot_id is None and "baseline_tagged_snapshot_id" in self.model_fields_set:
            _dict['baseline_tagged_snapshot_id'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of System from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created": obj.get("created"),
            "created_by_id": obj.get("created_by_id"),
            "name": obj.get("name"),
            "description": obj.get("description"),
            "archive_status": obj.get("archive_status"),
            "configurations": [SystemConfiguration.from_dict(_item) for _item in obj["configurations"]] if obj.get("configurations") is not None else None,
            "baseline_tagged_snapshot_id": obj.get("baseline_tagged_snapshot_id")
        })
        return _obj



# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from istari_digital_client.models.archive_status_name import ArchiveStatusName
from istari_digital_client.models.resource_type import ResourceType
from typing import Optional, Set
from typing_extensions import Self

class ResourceSearchItem(BaseModel):
    """
    Resource class for search results.
    """ # noqa: E501
    id: StrictStr
    file_id: StrictStr
    file_revision_id: StrictStr
    type_name: ResourceType
    created_by_id: StrictStr
    archive_status: ArchiveStatusName
    created: datetime
    updated: datetime
    name: Optional[StrictStr] = None
    extension: Optional[StrictStr] = None
    size: Optional[StrictInt] = None
    mime: Optional[StrictStr] = None
    description: Optional[StrictStr] = None
    version_name: Optional[StrictStr] = None
    external_identifier: Optional[StrictStr] = None
    display_name: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["id", "file_id", "file_revision_id", "type_name", "created_by_id", "archive_status", "created", "updated", "name", "extension", "size", "mime", "description", "version_name", "external_identifier", "display_name"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ResourceSearchItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict['name'] = None

        # set to None if extension (nullable) is None
        # and model_fields_set contains the field
        if self.extension is None and "extension" in self.model_fields_set:
            _dict['extension'] = None

        # set to None if size (nullable) is None
        # and model_fields_set contains the field
        if self.size is None and "size" in self.model_fields_set:
            _dict['size'] = None

        # set to None if mime (nullable) is None
        # and model_fields_set contains the field
        if self.mime is None and "mime" in self.model_fields_set:
            _dict['mime'] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict['description'] = None

        # set to None if version_name (nullable) is None
        # and model_fields_set contains the field
        if self.version_name is None and "version_name" in self.model_fields_set:
            _dict['version_name'] = None

        # set to None if external_identifier (nullable) is None
        # and model_fields_set contains the field
        if self.external_identifier is None and "external_identifier" in self.model_fields_set:
            _dict['external_identifier'] = None

        # set to None if display_name (nullable) is None
        # and model_fields_set contains the field
        if self.display_name is None and "display_name" in self.model_fields_set:
            _dict['display_name'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ResourceSearchItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "file_id": obj.get("file_id"),
            "file_revision_id": obj.get("file_revision_id"),
            "type_name": obj.get("type_name"),
            "created_by_id": obj.get("created_by_id"),
            "archive_status": obj.get("archive_status"),
            "created": obj.get("created"),
            "updated": obj.get("updated"),
            "name": obj.get("name"),
            "extension": obj.get("extension"),
            "size": obj.get("size"),
            "mime": obj.get("mime"),
            "description": obj.get("description"),
            "version_name": obj.get("version_name"),
            "external_identifier": obj.get("external_identifier"),
            "display_name": obj.get("display_name")
        })
        return _obj



import os
import logging
from logging import FileHandler
import multiprocessing
from pathlib import Path
from typing import Dict, List, Literal, Optional, TypedDict, Union
from typing_extensions import NotRequired
from dataclasses import field, dataclass
from functools import cached_property
import istari_digital_core

from istari_digital_client.env import env_bool, env_int, env_str, env_cache_root

logger = logging.getLogger("istari-digital-client.configuration")

JSON_SCHEMA_VALIDATION_KEYWORDS = {
    "multipleOf",
    "maximum",
    "exclusiveMaximum",
    "minimum",
    "exclusiveMinimum",
    "maxLength",
    "minLength",
    "pattern",
    "maxItems",
    "minItems",
}

ServerVariablesT = Dict[str, str]

BearerAuthSetting = TypedDict(
    "BearerAuthSetting",
    {
        "type": Literal["bearer"],
        "in": Literal["header"],
        "key": Literal["Authorization"],
        "value": str,
    },
)

AuthSettings = TypedDict(
    "AuthSettings",
    {
        "RequestAuthenticator": BearerAuthSetting,
    },
    total=False,
)


class HostSettingVariable(TypedDict):
    description: str
    default_value: str
    enum_values: List[str]


class HostSetting(TypedDict):
    url: str
    description: str
    variables: NotRequired[Dict[str, HostSettingVariable]]


@dataclass
class Configuration:
    registry_url: str | None = field(
        default_factory=env_str("ISTARI_REGISTRY_URL", default=None)
    )
    registry_auth_token: str | None = field(
        default_factory=env_str("ISTARI_REGISTRY_AUTH_TOKEN")
    )
    http_request_timeout_secs: int | None = field(
        default_factory=env_int("ISTARI_CLIENT_HTTP_REQUEST_TIMEOUT_SECS"),
    )
    retry_enabled: bool | None = field(
        default_factory=env_bool("ISTARI_CLIENT_RETRY_ENABLED", default=True)
    )
    retry_max_attempts: int | None = field(
        default_factory=env_int("ISTARI_CLIENT_RETRY_MAX_ATTEMPTS")
    )
    retry_min_interval_millis: int | None = field(
        default_factory=env_int("ISTARI_CLIENT_RETRY_MIN_INTERVAL_MILLIS")
    )
    retry_max_interval_millis: int | None = field(
        default_factory=env_int("ISTARI_CLIENT_RETRY_MAX_INTERVAL_MILLIS")
    )
    filesystem_cache_enabled: bool | None = field(
        default_factory=env_bool("ISTARI_CLIENT_FILESYSTEM_CACHE_ENABLED", default=True)
    )
    filesystem_cache_root: Path = field(
        default_factory=env_cache_root("ISTARI_CLIENT_FILESYSTEM_CACHE_ROOT")
    )
    filesystem_cache_clean_on_exit: bool = field(
        default_factory=env_bool(
            "ISTARI_CLIENT_FILESYSTEM_CACHE_CLEAN_BEFORE_EXIT", default=True
        )
    )
    retry_jitter_enabled: bool | None = field(
        default_factory=env_bool("ISTARI_CLIENT_RETRY_JITTER_ENABLED", default=True)
    )
    multipart_chunksize: int | None = field(
        default_factory=env_int("ISTARI_CLIENT_MULTIPART_CHUNKSIZE")
    )
    multipart_threshold: int | None = field(
        default_factory=env_int("ISTARI_CLIENT_MULTIPART_THRESHOLD")
    )
    server_index: Optional[int] = field(default=None)
    server_variables: Optional[ServerVariablesT] = field(default=None)
    server_operation_index: Optional[Dict[int, int]] = field(default_factory=lambda: {})
    server_operation_variables: Optional[Dict[int, ServerVariablesT]] = field(
        default_factory=lambda: {}
    )
    ignore_operation_servers: bool = field(default=False)
    ssl_ca_cert: Optional[str] = field(default=None)
    ca_cert_data: Optional[Union[str, bytes]] = field(default=None)
    debug: Optional[bool] = field(default=None)

    temp_folder_path: Optional[str] = field(init=False, default=None)
    logger: Dict[str, logging.Logger] = field(
        init=False,
        default_factory=lambda: {
            "package_logger": logging.getLogger("istari_digital_client.openapi_client"),
            "urllib3_logger": logging.getLogger("urllib3"),
        },
    )
    logger_format: str = field(
        init=False, default="%(asctime)s %(levelname)s %(message)s"
    )
    logger_stream_handler: Optional[logging.StreamHandler] = field(
        init=False, default=None
    )
    logger_file_handler: Optional[FileHandler] = field(init=False, default=None)
    logger_file: Optional[str] = field(init=False, default=None)
    verify_ssl: bool = field(init=False, default=True)
    cert_file: Optional[str] = field(init=False, default=None)
    key_file: Optional[str] = field(init=False, default=None)
    assert_hostname: Optional[bool] = field(init=False, default=None)
    tls_server_name: Optional[str] = field(init=False, default=None)
    connection_pool_maxsize: int = field(
        init=False, default=multiprocessing.cpu_count() * 5
    )
    proxy: Optional[str] = field(init=False, default=None)
    proxy_headers: Optional[Dict[str, str]] = field(init=False, default=None)
    safe_chars_for_path_param: str = field(init=False, default="")
    client_side_validation: bool = field(init=False, default=True)
    socket_options = None
    datetime_format: str = field(init=False, default="%Y-%m-%dT%H:%M:%S.%f%z")
    date_format: str = field(init=False, default="%Y-%m-%d")

    def __post_init__(self) -> None:
        self.server_index = (
            0
            if self.server_index is None and self.registry_url is None
            else self.server_index
        )
        os.environ["ISTARI_REGISTRY_URL"] = self.registry_url or ""
        logger.debug(
            "set os.environ['ISTARI_REGISTRY_URL'] to '%s'",
            os.environ.get("ISTARI_REGISTRY_URL"),
        )
        os.environ["ISTARI_REGISTRY_AUTH_TOKEN"] = self.registry_auth_token or ""
        logger.debug(
            "setting os.environ['ISTARI_REGISTRY_AUTH_TOKEN'] to '%s'",
            os.environ.get("ISTARI_REGISTRY_AUTH_TOKEN"),
        )

    def auth_settings(self) -> AuthSettings:
        """Gets Auth Settings dict for api client.

        :return: The Auth Settings information dict.
        """
        auth: AuthSettings = {}
        if self.registry_auth_token is not None:
            auth["RequestAuthenticator"] = {
                "type": "bearer",
                "in": "header",
                "key": "Authorization",
                "value": "Bearer " + self.registry_auth_token,
            }
        return auth

    @staticmethod
    def to_debug_report() -> str:
        """Gets the essential information for debugging.

        :return: The report for debugging.
        """
        return (
            "Python SDK Debug Report:\n" "OS: {env}\n" "Python Version: {pyversion}\n"
        )

    @staticmethod
    def get_host_settings() -> List[HostSetting]:
        """Gets an array of host settings

        :return: An array of host settings
        """
        return [
            {
                "url": "",
                "description": "No description provided",
            }
        ]

    def get_host_from_settings(
        self,
        index: Optional[int],
        variables: Optional[ServerVariablesT] = None,
        servers: Optional[List[HostSetting]] = None,
    ) -> str:
        """Gets host URL based on the index and variables
        :param index: array index of the host settings
        :param variables: hash of variable and the corresponding value
        :param servers: an array of host settings or None
        :return: URL based on host settings
        """
        if index is None:
            if not self.registry_url:
                raise ValueError(
                    "No server URL (registry_url) found and server_index is not defined."
                )
            return self.registry_url

        variables = {} if variables is None else variables
        servers = self.get_host_settings() if servers is None else servers

        try:
            server = servers[index]
        except IndexError:
            raise ValueError(
                "Invalid index {0} when selecting the host settings. "
                "Must be less than {1}".format(index, len(servers))
            )

        url = server["url"]

        # go through variables and replace placeholders
        for variable_name, variable in server.get("variables", {}).items():
            used_value = variables.get(variable_name, variable["default_value"])

            if "enum_values" in variable and used_value not in variable["enum_values"]:
                raise ValueError(
                    "The variable `{0}` in the host URL has invalid value "
                    "{1}. Must be {2}.".format(
                        variable_name, variables[variable_name], variable["enum_values"]
                    )
                )

            url = url.replace("{" + variable_name + "}", used_value)

        return url

    @classmethod
    def from_native_configuration(
        cls: type["Configuration"], native: istari_digital_core.Configuration
    ) -> "Configuration":
        return Configuration(
            registry_url=native.registry_url,
            registry_auth_token=native.registry_auth_token,
            retry_enabled=native.retry_enabled,  # type: ignore[attr-defined]
            retry_max_attempts=native.retry_max_attempts,  # type: ignore[attr-defined]
            retry_min_interval_millis=native.retry_min_interval_millis,  # type: ignore[attr-defined]
            retry_max_interval_millis=native.retry_max_interval_millis,  # type: ignore[attr-defined]
            retry_jitter_enabled=native.retry_jitter_enabled,
            multipart_chunksize=native.multipart_chunksize,
            multipart_threshold=native.multipart_threshold,
        )

    @cached_property
    def native_configuration(self) -> istari_digital_core.Configuration:
        return istari_digital_core.Configuration(
            registry_url=self.registry_url,
            registry_auth_token=self.registry_auth_token,
            retry_enabled=self.retry_enabled,  # type: ignore[attr-defined]
            retry_max_attempts=self.retry_max_attempts,  # type: ignore[attr-defined]
            retry_min_interval_millis=self.retry_min_interval_millis,  # type: ignore[attr-defined]
            retry_max_interval_millis=self.retry_max_interval_millis,  # type: ignore[attr-defined]
            retry_jitter_enabled=self.retry_jitter_enabled,
            multipart_chunksize=self.multipart_chunksize,
            multipart_threshold=self.multipart_threshold,
        )


class ConfigurationError(ValueError):
    pass



# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class FunctionAuthType(str, Enum):
    """
    FunctionAuthType
    """

    """
    allowed enum values
    """
    JWT = 'JWT'
    TOKEN = 'TOKEN'
    BASIC = 'BASIC'
    OAUTH1 = 'OAUTH1'
    OAUTH2 = 'OAUTH2'
    CAS_DELEGATED_3_DX = 'CAS_DELEGATED_3DX'
    OAUTH1_REGISTRATION = 'OAUTH1_REGISTRATION'
    OAUTH2_REGISTRATION = 'OAUTH2_REGISTRATION'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of FunctionAuthType from a JSON string"""
        return cls(json.loads(json_str))



import abc
from typing import List, Optional

from istari_digital_client.models.file import File
from istari_digital_client.models.file_revision import FileRevision
from istari_digital_client.models.file_revision_having import FileRevisionHaving


class FileHaving(FileRevisionHaving, abc.ABC):
    @property
    def _file(self) -> File:
        file = getattr(self, "file", None)

        if file is None:
            raise ValueError("file is not set")

        return file

    @property
    def revisions(self) -> List[FileRevision]:
        return self._file.revisions

    @property
    def revision(self) -> FileRevision:
        return self.revisions[-1]

    def update_properties(
        self,
        description: Optional[str] = None,
        display_name: Optional[str] = None,
        external_identifier: Optional[str] = None,
        version_name: Optional[str] = None,
    ) -> File:
        return self._file.update_properties(description, display_name, external_identifier, version_name)

    def update_description(self, description: str) -> File:
        return self._file.update_description(description)

    def update_display_name(self, display_name: str) -> File:
        return self._file.update_display_name(display_name)

    def update_external_identifier(self, external_identifier: str) -> File:
        return self._file.update_external_identifier(external_identifier)

    def update_version_name(self, version_name: str) -> File:
        return self._file.update_version_name(version_name)



import abc
from pydantic import BaseModel

from istari_digital_client.models.file_revision import FileRevision
from istari_digital_client.models.readable import Readable
from istari_digital_client.models.properties import Properties


class FileRevisionHaving(BaseModel, Readable, abc.ABC):
    @property
    @abc.abstractmethod
    def revision(self) -> FileRevision: ...

    def read_bytes(self) -> bytes:
        return self.revision.read_bytes()

    @property
    def properties(self) -> Properties:
        return self.revision.properties

    @property
    def extension(self) -> str | None:
        return self.revision.extension

    @property
    def name(self) -> str | None:
        if self.revision.name is None or self.extension is None:
            return None

        file_name = self.revision.name
        if file_name.lower().endswith(f".{self.extension}"):
            return file_name
        return ".".join([file_name, self.extension])

    @property
    def stem(self) -> str | None:
        return self.revision.stem

    @property
    def suffix(self) -> str | None:
        return self.revision.suffix

    @property
    def description(self) -> str | None:
        return self.revision.description

    @property
    def size(self) -> int | None:
        return self.revision.size

    @property
    def mime(self) -> str | None:
        return self.revision.mime

    @property
    def version_name(self) -> str | None:
        return self.revision.version_name

    @property
    def external_identifier(self) -> str | None:
        return self.revision.external_identifier

    @property
    def display_name(self) -> str | None:
        return self.revision.display_name



# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class AgentStatusName(str, Enum):
    """
    AgentStatusName
    """

    """
    allowed enum values
    """
    IDLE = 'Idle'
    CLAIMINGJOB = 'ClaimingJob'
    EXECUTINGJOB = 'ExecutingJob'
    EXECUTIONFAILED = 'ExecutionFailed'
    EXECUTIONSUCCESS = 'ExecutionSuccess'
    UPDATINGAGENT = 'UpdatingAgent'
    SUCCESSFULAGENTUPDATE = 'SuccessfulAgentUpdate'
    FAILEDAGENTUPDATE = 'FailedAgentUpdate'
    UPDATINGMODULE = 'UpdatingModule'
    SUCCESSFULMODULEUPDATE = 'SuccessfulModuleUpdate'
    FAILEDMODULEUPDATE = 'FailedModuleUpdate'
    PAUSED = 'Paused'
    UNKNOWN = 'Unknown'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of AgentStatusName from a JSON string"""
        return cls(json.loads(json_str))



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from typing import Any, Dict, Optional, Set
from typing_extensions import Self

from istari_digital_client.models.pageable import Pageable
from istari_digital_client.models.client_having import ClientHaving
from istari_digital_client.models.snapshot_revision_search_item import SnapshotRevisionSearchItem

class PageSnapshotRevisionSearchItem(Pageable[SnapshotRevisionSearchItem], ClientHaving):
    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PageSnapshotRevisionSearchItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in items (list)
        _items = []
        if self.items:
            for _item_items in self.items:
                if _item_items:
                    _items.append(_item_items.to_dict())
            _dict['items'] = _items
        # set to None if total (nullable) is None
        # and model_fields_set contains the field
        if self.total is None and "total" in self.model_fields_set:
            _dict['total'] = None

        # set to None if page (nullable) is None
        # and model_fields_set contains the field
        if self.page is None and "page" in self.model_fields_set:
            _dict['page'] = None

        # set to None if size (nullable) is None
        # and model_fields_set contains the field
        if self.size is None and "size" in self.model_fields_set:
            _dict['size'] = None

        # set to None if pages (nullable) is None
        # and model_fields_set contains the field
        if self.pages is None and "pages" in self.model_fields_set:
            _dict['pages'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PageSnapshotRevisionSearchItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "items": [SnapshotRevisionSearchItem.from_dict(_item) for _item in obj["items"]] if obj.get("items") is not None else None,
            "total": obj.get("total"),
            "page": obj.get("page"),
            "size": obj.get("size"),
            "pages": obj.get("pages")
        })
        return _obj



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List
from istari_digital_client.models.agent_display_name import AgentDisplayName
from istari_digital_client.models.agent_information import AgentInformation
from istari_digital_client.models.agent_modules import AgentModules
from istari_digital_client.models.agent_status import AgentStatus
from typing import Optional, Set
from typing_extensions import Self


class Agent(BaseModel):
    """
    Agent
    """ # noqa: E501
    id: StrictStr
    created: datetime
    agent_identifier: StrictStr
    display_name_history: Optional[List[AgentDisplayName]] = None
    information_history: Optional[List[AgentInformation]] = None
    status_history: Optional[List[AgentStatus]] = None
    modules_history: Optional[List[AgentModules]] = None
    created_by_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["id", "created", "agent_identifier", "information_history", "status_history", "modules_history", "created_by_id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    @property
    def display_name(self) -> Optional[AgentDisplayName]:
        """Returns the latest display name record"""
        return self.display_name_history[-1] if self.display_name_history else None

    @property
    def information(self) -> Optional[AgentInformation]:
        """Returns the latest information record"""
        return self.information_history[-1] if self.information_history else None

    @property
    def agent_version(self) -> Optional[str]:
        """Returns the latest agent version"""
        return self.information.agent_version if self.information else None

    @property
    def host_os(self) -> Optional[str]:
        """Returns the latest host OS"""
        return self.information.host_os if self.information else None

    @property
    def status(self) -> Optional[AgentStatus]:
        """Returns the latest status record"""
        return self.status_history[-1] if self.status_history else None

    @property
    def modules(self) -> Optional[AgentModules]:
        """Returns the latest modules record"""
        return self.modules_history[-1] if self.modules_history else None

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Agent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in display_name_history (list)
        _items = []
        if self.display_name_history:
            for _item_display_name_history in self.display_name_history:
                if _item_display_name_history:
                    _items.append(_item_display_name_history.to_dict())
            _dict['display_name_history'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in information_history (list)
        _items = []
        if self.information_history:
            for _item_information_history in self.information_history:
                if _item_information_history:
                    _items.append(_item_information_history.to_dict())
            _dict['information_history'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in status_history (list)
        _items = []
        if self.status_history:
            for _item_status_history in self.status_history:
                if _item_status_history:
                    _items.append(_item_status_history.to_dict())
            _dict['status_history'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in modules_history (list)
        _items = []
        if self.modules_history:
            for _item_modules_history in self.modules_history:
                if _item_modules_history:
                    _items.append(_item_modules_history.to_dict())
            _dict['modules_history'] = _items
        # set to None if created_by_id (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_id is None and "created_by_id" in self.model_fields_set:
            _dict['created_by_id'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Agent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created": obj.get("created"),
            "agent_identifier": obj.get("agent_identifier"),
            "display_name_history": [AgentDisplayName.from_dict(_item) for _item in obj["display_name_history"]] if obj.get("display_name_history") is not None else None,
            "information_history": [AgentInformation.from_dict(_item) for _item in obj["information_history"]] if obj.get("information_history") is not None else None,
            "status_history": [AgentStatus.from_dict(_item) for _item in obj["status_history"]] if obj.get("status_history") is not None else None,
            "modules_history": [AgentModules.from_dict(_item) for _item in obj["modules_history"]] if obj.get("modules_history") is not None else None,
            "created_by_id": obj.get("created_by_id")
        })
        return _obj



from __future__ import annotations
import pprint
import re  # noqa: F401
import json
import uuid
import hashlib
import logging
import traceback
import tempfile
from threading import Lock
from pathlib import Path

from datetime import datetime, timezone
from functools import cached_property

from pydantic import BaseModel, ConfigDict, StrictInt, StrictStr, PrivateAttr
from typing import Any, Dict, List, Optional, Set, TYPE_CHECKING, ClassVar
from typing_extensions import Self

from istari_digital_client.models.file_revision_archive_status import (
    FileRevisionArchiveStatus,
)
from istari_digital_client.models.archive_status_name import ArchiveStatusName
from istari_digital_client.models.product import Product
from istari_digital_client.models.source import Source
from istari_digital_client.models.token import Token
from istari_digital_client.models.properties import Properties

from istari_digital_client.models.readable import Readable
from istari_digital_client.models.archivable import Archivable
from istari_digital_client.models.client_having import ClientHaving

import istari_digital_core

if TYPE_CHECKING:
    from istari_digital_client.models.file import File
    from istari_digital_client.api.client_api import Resource

logger = logging.getLogger("istari-digital-client.token_reader")


class ReadError(IOError):
    pass


class CacheError(Exception):
    pass


class InvalidChecksumError(ValueError):
    pass


class FileRevision(BaseModel, ClientHaving, Readable, Archivable):
    """
    Represents a file revision object.  This class inherits from the Base classes.
    """  # noqa: E501

    id: StrictStr
    created: datetime
    file_id: Optional[StrictStr]
    content_token: Token
    properties_token: Token
    archive_status_history: List[FileRevisionArchiveStatus]
    name: Optional[StrictStr] = None
    stem: Optional[StrictStr] = None
    suffix: Optional[StrictStr] = None
    extension: Optional[StrictStr] = None
    description: Optional[StrictStr] = None
    size: Optional[StrictInt] = None
    mime: Optional[StrictStr] = None
    version_name: Optional[StrictStr] = None
    external_identifier: Optional[StrictStr] = None
    display_name: Optional[StrictStr] = None
    sources: Optional[List[Source]] = None
    products: Optional[List[Product]] = None
    created_by_id: Optional[StrictStr] = None

    # Private attributes
    _cache_path_lock_timeout: int = PrivateAttr(default=2)
    _filesystem_cache_hits: int = PrivateAttr(default=0)
    _filesystem_cache_misses: int = PrivateAttr(default=0)
    _filesystem_cache_puts: int = PrivateAttr(default=0)

    __client_fields__: ClassVar[List[str]] = ["sources", "products"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    def __post_init__(self):
        self.stem = Path(self.name).stem if self.name else None
        self.suffix = Path(self.name).suffix if self.name else None

    @property
    def file(self) -> Optional["File"]:
        if self.file_id is None:
            raise ReadError("File ID is not set")
        if self.client is None:
            raise ReadError("Client is not set")
        return self.client.get_file(self.file_id)

    @property
    def resource(self) -> Optional["Resource"]:
        file = self.file
        if file is None or self.client is None or file.resource_type is None or file.resource_id is None:
            return None

        return self.client.get_resource(file.resource_type, file.resource_id)

    def source_revision_ids(self) -> Optional[list[str]]:
        if not self.sources:
            return None
        return [source.revision_id for source in self.sources]

    def source_product_ids(self) -> Optional[list[str]]:
        if not self.products:
            return None
        return [product.revision_id for product in self.products]

    @property
    def archive_status(self) -> FileRevisionArchiveStatus:
        return self.archive_status_history[-1]

    @property
    def properties(self) -> Properties:
        if self.client is None:
            raise ReadError("Client is not set")
        return self.client.read_properties(self.properties_token)

    def read_bytes(self) -> bytes:
        if self.client is None:
            raise ReadError("Client is not set")
        if self.client.config.filesystem_cache_enabled:
            return self._filesystem_caching_read_bytes()
        return self.client.read_contents(self.content_token)

    def __del__(self):
        """Delete cached content."""
        if self._cache_path is not None:
            self._cache_path.unlink(missing_ok=True)

    @cached_property
    def _cache_path_lock(self) -> Lock:
        return Lock()

    @cached_property
    def _log_msg_pfx(self) -> str:
        return f"token {self.id} -"

    def _cache_identifier(self, size: int = 16) -> str:
        _hash = hashlib.shake_256()
        _hash.update(self.content_token.sha.encode("utf-8"))
        _hash.update(self.content_token.salt.encode("utf-8"))
        return _hash.hexdigest(size)

    @cached_property
    def _cache_dir(self) -> Path:
        if self.client is None:
            raise CacheError("Client is not set")
        subdir = self._cache_identifier(2)
        _dir = self.client.config.filesystem_cache_root / subdir
        _dir.mkdir(mode=0o700, parents=True, exist_ok=True)
        return _dir

    @cached_property
    def _cache_name(self) -> str:
        return self._cache_identifier(32)

    @cached_property
    def _cache_path(self) -> Optional[Path]:
        if self.client is None:
            return None
        return self._cache_dir / self._cache_name

    def _cache_dir_mktemp(self) -> Path:
        return Path(
            tempfile.mktemp(
                suffix=str(uuid.uuid4()) + ".tmp",
                prefix=self._cache_name,
                dir=self._cache_dir,
            )
        )

    def _checksum_verified(self, data: bytes) -> bytes:
        if self._cache_path is None:
            raise CacheError("Cache path is not set")
        data = self._cache_path.read_bytes()
        _hash = hashlib.sha384()
        _hash.update(data)
        _hash.update(self.content_token.salt.encode("utf-8"))
        actual = _hash.hexdigest()
        expected = self.content_token.sha
        if not actual == expected:
            msg = f"Token data content checksum is invalid ({actual} != {expected})"
            raise InvalidChecksumError(msg)
        return data

    def _checksum_verified_cache_read(self) -> bytes:
        if self._cache_path is None:
            raise CacheError("Cache path is not set")
        if not self._cache_path.exists():
            raise FileNotFoundError(self._cache_path)
        data = self._checksum_verified(self._cache_path.read_bytes())
        return data

    def _filesystem_caching_read_bytes(self) -> bytes:
        if self.client is None:
            raise ReadError("Client is not set")
        if self._cache_path is None:
            raise CacheError("Cache path is not set")
        with self._cache_path_lock:
            try:
                data = self._checksum_verified_cache_read()
                self._filesystem_cache_hits += 1
                return data
            except Exception:
                self._filesystem_cache_misses += 1
                logger.debug("%s", traceback.format_exc())
                if not self._cache_dir.exists():
                    self._cache_dir.mkdir(mode=0o700, parents=True, exist_ok=True)
                if self._cache_path.exists():
                    self._cache_path.unlink(missing_ok=True)
                temp_path = Path(self._cache_dir_mktemp())
                logger.debug("%s downloading to %s", self._log_msg_pfx, temp_path)
                data = self.client.read_contents(self.content_token)
                temp_path.write_bytes(data)
                size = temp_path.stat().st_size
                temp_path.replace(self._cache_path)
                logger.debug(
                    "%s downloaded contents to filesystem cache (size: %d): %s",
                    self._log_msg_pfx,
                    size,
                    self._cache_path,
                )
                self._filesystem_cache_puts += 1
                return self._checksum_verified(data)

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of FileRevision from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of content_token
        if self.content_token:
            _dict["content_token"] = self.content_token.to_dict()
        # override the default output from pydantic by calling `to_dict()` of properties_token
        if self.properties_token:
            _dict["properties_token"] = self.properties_token.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in archive_status_history (list)
        _items = []
        if self.archive_status_history:
            for _item_archive_status_history in self.archive_status_history:
                if _item_archive_status_history:
                    _items.append(_item_archive_status_history.to_dict())
            _dict["archive_status_history"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in sources (list)
        _items = []
        if self.sources:
            for _item_sources in self.sources:
                if _item_sources:
                    _items.append(_item_sources.to_dict())
            _dict["sources"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in products (list)
        _items = []
        if self.products:
            for _item_products in self.products:
                if _item_products:
                    _items.append(_item_products.to_dict())
            _dict["products"] = _items
        # set to None if file_id (nullable) is None
        # and model_fields_set contains the field
        if self.file_id is None and "file_id" in self.model_fields_set:
            _dict["file_id"] = None

        # set to None if name (nullable) is None
        # and model_fields_set contains the field
        if self.name is None and "name" in self.model_fields_set:
            _dict["name"] = None

        # set to None if extension (nullable) is None
        # and model_fields_set contains the field
        if self.extension is None and "extension" in self.model_fields_set:
            _dict["extension"] = None

        # set to None if size (nullable) is None
        # and model_fields_set contains the field
        if self.size is None and "size" in self.model_fields_set:
            _dict["size"] = None

        # set to None if description (nullable) is None
        # and model_fields_set contains the field
        if self.description is None and "description" in self.model_fields_set:
            _dict["description"] = None

        # set to None if mime (nullable) is None
        # and model_fields_set contains the field
        if self.mime is None and "mime" in self.model_fields_set:
            _dict["mime"] = None

        # set to None if version_name (nullable) is None
        # and model_fields_set contains the field
        if self.version_name is None and "version_name" in self.model_fields_set:
            _dict["version_name"] = None

        # set to None if external_identifier (nullable) is None
        # and model_fields_set contains the field
        if (
            self.external_identifier is None
            and "external_identifier" in self.model_fields_set
        ):
            _dict["external_identifier"] = None

        # set to None if display_name (nullable) is None
        # and model_fields_set contains the field
        if self.display_name is None and "display_name" in self.model_fields_set:
            _dict["display_name"] = None

        # set to None if created_by_id (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_id is None and "created_by_id" in self.model_fields_set:
            _dict["created_by_id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of FileRevision from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "created": obj.get("created"),
                "file_id": obj.get("file_id"),
                "content_token": Token.from_dict(obj["content_token"])
                if obj.get("content_token") is not None
                else None,
                "properties_token": Token.from_dict(obj["properties_token"])
                if obj.get("properties_token") is not None
                else None,
                "archive_status_history": [
                    FileRevisionArchiveStatus.from_dict(_item)
                    for _item in obj["archive_status_history"]
                ]
                if obj.get("archive_status_history") is not None
                else None,
                "name": obj.get("name"),
                "extension": obj.get("extension"),
                "size": obj.get("size"),
                "description": obj.get("description"),
                "mime": obj.get("mime"),
                "version_name": obj.get("version_name"),
                "external_identifier": obj.get("external_identifier"),
                "display_name": obj.get("display_name"),
                "sources": [Source.from_dict(_item) for _item in obj["sources"]]
                if obj.get("sources") is not None
                else None,
                "products": [Product.from_dict(_item) for _item in obj["products"]]
                if obj.get("products") is not None
                else None,
                "created_by_id": obj.get("created_by_id"),
            }
        )
        _obj.stem = Path(_obj.name).stem if _obj.name else None
        _obj.suffix = Path(_obj.name).suffix if _obj.name else None

        return _obj

    @classmethod
    def from_storage_revision(
        cls,
        storage_revision: istari_digital_core.Revision,
        sources: Optional[List[Source]],
    ) -> Self:
        file_revision_id = str(uuid.uuid4())

        file_revision_archive_status = FileRevisionArchiveStatus(
            id=str(uuid.uuid4()),
            created=datetime.now(timezone.utc),
            name=ArchiveStatusName.ACTIVE,
            reason="Initial",
            created_by_id=None,
            file_revision_id=file_revision_id,
        )

        return cls(
            id=file_revision_id,
            created=datetime.now(timezone.utc),
            file_id=None,
            content_token=Token.from_storage_token(storage_revision.content_token),
            properties_token=Token.from_storage_token(storage_revision.properties_token),
            archive_status_history=[file_revision_archive_status],
            name=storage_revision.properties.file_name,
            extension=storage_revision.properties.extension,
            size=storage_revision.properties.size,
            description=storage_revision.properties.description,
            mime=storage_revision.properties.mime,
            version_name=storage_revision.properties.version_name,
            external_identifier=storage_revision.properties.external_identifier,
            display_name=storage_revision.properties.display_name,
            sources=sources,
            products=None,
            created_by_id=None,
        )



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Set, TYPE_CHECKING
from typing_extensions import Self

from istari_digital_client.models.control_tag import ControlTag
from istari_digital_client.models.file_archive_status import FileArchiveStatus
from istari_digital_client.models.file_revision import FileRevision
from istari_digital_client.models.file_revision_having import FileRevisionHaving
from istari_digital_client.models.shareable import Shareable
from istari_digital_client.models.archivable import Archivable
from istari_digital_client.models.client_having import ClientHaving

if TYPE_CHECKING:
    from istari_digital_client.api.client_api import Resource


class File(ClientHaving, FileRevisionHaving, Shareable, Archivable):
    """
    A class representing a file.
    """  # noqa: E501

    id: StrictStr
    created: datetime
    revisions: List[FileRevision]
    archive_status_history: List[FileArchiveStatus]
    resource_id: Optional[StrictStr] = None
    resource_type: Optional[StrictStr] = None
    created_by_id: Optional[StrictStr] = None
    control_tags: Optional[List[ControlTag]] = None
    __client_fields__: ClassVar[List[str]] = ["revisions"]
    __properties: ClassVar[List[str]] = [
        "id",
        "created",
        "revisions",
        "archive_status_history",
        "resource_id",
        "resource_type",
        "created_by_id",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    @property
    def revision(self) -> FileRevision:
        return self.revisions[-1]

    @property
    def resource(self) -> "Resource":
        if self.client is None:
            raise ValueError("client is not set")
        if self.resource_id is None:
            raise ValueError("resource_id is not set")
        if self.resource_type is None:
            raise ValueError("resource_type is not set")
        return self.client.get_resource(self.resource_type, self.resource_id)

    @property
    def archive_status(self) -> FileArchiveStatus:
        return self.archive_status_history[-1]

    def update_properties(
        self,
        description: Optional[str] = None,
        display_name: Optional[str] = None,
        external_identifier: Optional[str] = None,
        version_name: Optional[str] = None,
    ) -> "File":
        if self.client is None:
            raise ValueError("client is not set")

        return self.client.update_revision_properties(
            self.revision,
            description=description,
            display_name=display_name,
            external_identifier=external_identifier,
            version_name=version_name,
        )

    def update_description(self, description: str) -> "File":
        return self.update_properties(
            description=description,
        )

    def update_display_name(self, display_name: str) -> "File":
        return self.update_properties(
            display_name=display_name,
        )

    def update_external_identifier(self, external_identifier: str) -> "File":
        return self.update_properties(
            external_identifier=external_identifier,
        )

    def update_version_name(self, version_name: str) -> "File":
        return self.update_properties(
            version_name=version_name,
        )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of File from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in revisions (list)
        _items = []
        if self.revisions:
            for _item_revisions in self.revisions:
                if _item_revisions:
                    _items.append(_item_revisions.to_dict())
            _dict["revisions"] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in archive_status_history (list)
        _items = []
        if self.archive_status_history:
            for _item_archive_status_history in self.archive_status_history:
                if _item_archive_status_history:
                    _items.append(_item_archive_status_history.to_dict())
            _dict["archive_status_history"] = _items
        # set to None if resource_id (nullable) is None
        # and model_fields_set contains the field
        if self.resource_id is None and "resource_id" in self.model_fields_set:
            _dict["resource_id"] = None

        # set to None if resource_type (nullable) is None
        # and model_fields_set contains the field
        if self.resource_type is None and "resource_type" in self.model_fields_set:
            _dict["resource_type"] = None

        # set to None if created_by_id (nullable) is None
        # and model_fields_set contains the field
        if self.created_by_id is None and "created_by_id" in self.model_fields_set:
            _dict["created_by_id"] = None

        # override the default output from pydantic by calling `to_dict()` of each item in control_tags (list)
        _items = []
        if self.control_tags:
            for _item_control_tags in self.control_tags:
                if _item_control_tags:
                    _items.append(_item_control_tags.to_dict())
            _dict['control_tags'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of File from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "id": obj.get("id"),
                "created": obj.get("created"),
                "revisions": [
                    FileRevision.from_dict(_item) for _item in obj["revisions"]
                ]
                if obj.get("revisions") is not None
                else None,
                "archive_status_history": [
                    FileArchiveStatus.from_dict(_item)
                    for _item in obj["archive_status_history"]
                ]
                if obj.get("archive_status_history") is not None
                else None,
                "resource_id": obj.get("resource_id"),
                "resource_type": obj.get("resource_type"),
                "created_by_id": obj.get("created_by_id"),
                "control_tags": [ControlTag.from_dict(_item) for _item in obj["control_tags"]] if obj.get("control_tags") is not None else None
            }
        )
        return _obj



import os
import abc
import json
from typing import TypeAlias, Union
from pydantic import BaseModel

from pathlib import Path
JSON: TypeAlias = dict[str, "JSON"] | list["JSON"] | str | int | float | bool | None
PathLike = Union[str, os.PathLike, Path]


class Readable(abc.ABC):
    @abc.abstractmethod
    def read_bytes(self) -> bytes: ...

    def read_text(self, encoding: str = "utf-8") -> str:
        return self.read_bytes().decode(encoding)

    def copy_to(self, dest: PathLike) -> Path:
        dest_path = Path(str(dest))
        dest_path.write_bytes(self.read_bytes())
        return dest_path

    def read_json(self, encoding: str = "utf-8") -> JSON:
        return json.loads(self.read_text(encoding=encoding))



import os
import tempfile
from pathlib import Path
from typing import Callable


def env_int(env_var: str, default: int | None = None) -> Callable[[], int | None]:
    def getter():
        return os.environ.get(env_var, default)

    return getter


def env_str(env_var: str, default: str | None = None) -> Callable[[], str | None]:
    def getter() -> str | None:
        val = os.environ.get(env_var, default)
        return val

    return getter


def env_bool(env_var: str, default: bool | None = None) -> Callable[[], bool]:
    truthy = {"true", "t", "1", "yes", "y"}
    falsy = {"false" "f", "0", "no", "n"}

    def getter() -> bool:
        val = os.environ.get(env_var, None)
        if val is None:
            if default is not None:
                return default
        else:
            lc_val = val.lower()
            if lc_val in truthy:
                return True
            if lc_val in falsy:
                return False
        raise ValueError(
            "env var '{env_var}' contains invalid literal for boolean: '{value}' and no default set"
        )

    return getter


def env_cache_root(env_var: str) -> Callable[[], Path]:
    def getter() -> Path:
        env_val = os.environ.get(env_var, None)
        if env_val:
            dir = Path(env_val)
            if not dir.is_dir():
                raise NotADirectoryError(dir)
            return dir
        else:
            dir = Path(tempfile.mkdtemp(prefix="istari-client-cache-"))
            dir.mkdir(parents=True, exist_ok=True)
            return dir

    return getter



from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime

from pydantic import ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Set, TYPE_CHECKING
from typing_extensions import Self

from istari_digital_client.models.control_tag import ControlTag
from istari_digital_client.models.comment import Comment
from istari_digital_client.models.file import File
from istari_digital_client.models.file_having import FileHaving
from istari_digital_client.models.resource_archive_status import ResourceArchiveStatus
from istari_digital_client.models.shareable import Shareable
from istari_digital_client.models.archivable import Archivable
from istari_digital_client.models.client_having import ClientHaving

if TYPE_CHECKING:
    from istari_digital_client.models.model import Model


class Artifact(ClientHaving, FileHaving, Shareable, Archivable):
    """
    Represents an artifact with file and model information.  Inherits from Base and FileReaderMixin.
    """ # noqa: E501
    id: StrictStr
    created: datetime
    file: File
    comments: List[Comment]
    archive_status_history: List[ResourceArchiveStatus]
    created_by_id: StrictStr
    model_id: Optional[StrictStr]
    control_tags: Optional[List[ControlTag]] = None
    __client_fields__: ClassVar[List[str]] = ["file", "comments"]
    __properties: ClassVar[List[str]] = ["id", "created", "file", "comments", "archive_status_history", "created_by_id", "model_id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )

    @property
    def archive_status(self) -> ResourceArchiveStatus:
        return self.archive_status_history[-1]

    @property
    def model(self) -> Optional["Model"]:
        if not self.model_id or not self.client:
            return None
        return self.client.get_model(self.model_id)

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Artifact from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of file
        if self.file:
            _dict['file'] = self.file.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in comments (list)
        _items = []
        if self.comments:
            for _item_comments in self.comments:
                if _item_comments:
                    _items.append(_item_comments.to_dict())
            _dict['comments'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in archive_status_history (list)
        _items = []
        if self.archive_status_history:
            for _item_archive_status_history in self.archive_status_history:
                if _item_archive_status_history:
                    _items.append(_item_archive_status_history.to_dict())
            _dict['archive_status_history'] = _items
        # set to None if model_id (nullable) is None
        # and model_fields_set contains the field
        if self.model_id is None and "model_id" in self.model_fields_set:
            _dict['model_id'] = None

        # override the default output from pydantic by calling `to_dict()` of each item in control_tags (list)
        _items = []
        if self.control_tags:
            for _item_control_tags in self.control_tags:
                if _item_control_tags:
                    _items.append(_item_control_tags.to_dict())
            _dict['control_tags'] = _items

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Artifact from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created": obj.get("created"),
            "file": File.from_dict(obj["file"]) if obj.get("file") is not None else None,
            "comments": [Comment.from_dict(_item) for _item in obj["comments"]] if obj.get("comments") is not None else None,
            "archive_status_history": [ResourceArchiveStatus.from_dict(_item) for _item in obj["archive_status_history"]] if obj.get("archive_status_history") is not None else None,
            "created_by_id": obj.get("created_by_id"),
            "model_id": obj.get("model_id"),
            "control_tags": [ControlTag.from_dict(_item) for _item in obj["control_tags"]] if obj.get("control_tags") is not None else None
        })

        return _obj



import abc

import inflection
from pydantic import StrictStr
from typing import TYPE_CHECKING, Optional, Callable

from istari_digital_client.models.archive import Archive
from istari_digital_client.models.restore import Restore

if TYPE_CHECKING:
    from istari_digital_client.api.client_api import ClientApi

class Archivable(abc.ABC):
    def _require_client(self) -> "ClientApi":
        """Internal helper to assert that a client exists."""
        client = getattr(self, "client", None)
        if client is None:
            raise ValueError(f"`client` is not set for instance of {self.__class__.__name__}")
        return client

    @property
    def _id(self) -> StrictStr:
        """The ID of the item to be archived or restored."""
        item_id = getattr(self, "id", None)

        if item_id is None:
            raise ValueError("id is not set")

        return item_id

    def archive(self, archive_reason: Optional[str] = None):
        """
        Archive the item using the appropriate client method based on its type.

        Args:
            archive_reason (Optional[str]): Reason for archiving.

        Raises:
            ValueError: If `client` is not set or class type is not archivable.
        """
        client = self._require_client()
        reason = Archive(reason=archive_reason) if archive_reason else None
        method_name = f"archive_{inflection.underscore(self.__class__.__name__)}"
        method: Callable[[StrictStr, Optional[Archive]], object] = getattr(client, method_name)

        try:
            return method(self._id, reason)
        except ValueError:
            raise ValueError(f"Cannot archive {self.__class__.__name__} with id {self._id}. Ensure the client is set and the item is archivable.")

    def restore(self, restore_reason: Optional[str] = None):
        """
        Restore the item using the appropriate client method based on its type.

        Args:
            restore_reason (Optional[str]): Reason for restoring.

        Raises:
            ValueError: If `client` is not set or class type is not restorable.
        """
        client = self._require_client()
        reason = Restore(reason=restore_reason) if restore_reason else None
        method_name = f"restore_{inflection.underscore(self.__class__.__name__)}"
        method: Callable[[StrictStr, Optional[Restore]], object] = getattr(client, method_name)

        try:
            return method(self._id, reason)
        except ValueError:
            raise ValueError(f"Cannot restore {self.__class__.__name__} with id {self._id}. Ensure the client is set and the item is restorable.")



# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from istari_digital_client.models.access_relation import AccessRelation
from istari_digital_client.models.access_resource_type import AccessResourceType
from istari_digital_client.models.access_subject_type import AccessSubjectType
from istari_digital_client.models.subject_info import SubjectInfo
from typing import Optional, Set
from typing_extensions import Self

class AccessRelationship(BaseModel):
    """
    PUBLIC: An access relationship that can be viewed/modified by a user of Istari.
    """ # noqa: E501
    subject_type: AccessSubjectType
    subject_id: StrictStr
    relation: AccessRelation
    resource_type: AccessResourceType
    resource_id: StrictStr
    subject_info: Optional[SubjectInfo] = None
    __properties: ClassVar[List[str]] = ["subject_type", "subject_id", "relation", "resource_type", "resource_id", "subject_info"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AccessRelationship from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of subject_info
        if self.subject_info:
            _dict['subject_info'] = self.subject_info.to_dict()
        # set to None if subject_info (nullable) is None
        # and model_fields_set contains the field
        if self.subject_info is None and "subject_info" in self.model_fields_set:
            _dict['subject_info'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AccessRelationship from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "subject_type": obj.get("subject_type"),
            "subject_id": obj.get("subject_id"),
            "relation": obj.get("relation"),
            "resource_type": obj.get("resource_type"),
            "resource_id": obj.get("resource_id"),
            "subject_info": SubjectInfo.from_dict(obj["subject_info"]) if obj.get("subject_info") is not None else None
        })
        return _obj



# coding: utf-8

"""
    File Service v2

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from istari_digital_client.models.control_tag import ControlTag
from istari_digital_client.models.personal_access_token import PersonalAccessToken
from istari_digital_client.models.user_state import UserState
from istari_digital_client.models.user_type import UserType
from typing import Optional, Set
from typing_extensions import Self

class User(BaseModel):
    """
    User Dataclass that combines info from UserEntity and UserInfo.
    """ # noqa: E501
    id: StrictStr
    created: datetime
    provider_name: StrictStr
    provider_user_id: StrictStr
    user_type: UserType
    personal_access_tokens: List[PersonalAccessToken]
    provider_user_state: Optional[UserState] = None
    user_name: Optional[StrictStr] = None
    display_name: Optional[StrictStr] = None
    first_name: Optional[StrictStr] = None
    last_name: Optional[StrictStr] = None
    email: Optional[StrictStr] = None
    machine_name: Optional[StrictStr] = None
    machine_description: Optional[StrictStr] = None
    control_tags: Optional[List[ControlTag]] = None
    provider_tenant_id: Optional[StrictStr] = None
    tenant_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = ["id", "created", "provider_name", "provider_user_id", "user_type", "personal_access_tokens", "provider_user_state", "user_name", "display_name", "first_name", "last_name", "email", "machine_name", "machine_description", "control_tags", "provider_tenant_id", "tenant_id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of User from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in personal_access_tokens (list)
        _items = []
        if self.personal_access_tokens:
            for _item_personal_access_tokens in self.personal_access_tokens:
                if _item_personal_access_tokens:
                    _items.append(_item_personal_access_tokens.to_dict())
            _dict['personal_access_tokens'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in control_tags (list)
        _items = []
        if self.control_tags:
            for _item_control_tags in self.control_tags:
                if _item_control_tags:
                    _items.append(_item_control_tags.to_dict())
            _dict['control_tags'] = _items
        # set to None if provider_user_state (nullable) is None
        # and model_fields_set contains the field
        if self.provider_user_state is None and "provider_user_state" in self.model_fields_set:
            _dict['provider_user_state'] = None

        # set to None if user_name (nullable) is None
        # and model_fields_set contains the field
        if self.user_name is None and "user_name" in self.model_fields_set:
            _dict['user_name'] = None

        # set to None if display_name (nullable) is None
        # and model_fields_set contains the field
        if self.display_name is None and "display_name" in self.model_fields_set:
            _dict['display_name'] = None

        # set to None if first_name (nullable) is None
        # and model_fields_set contains the field
        if self.first_name is None and "first_name" in self.model_fields_set:
            _dict['first_name'] = None

        # set to None if last_name (nullable) is None
        # and model_fields_set contains the field
        if self.last_name is None and "last_name" in self.model_fields_set:
            _dict['last_name'] = None

        # set to None if email (nullable) is None
        # and model_fields_set contains the field
        if self.email is None and "email" in self.model_fields_set:
            _dict['email'] = None

        # set to None if machine_name (nullable) is None
        # and model_fields_set contains the field
        if self.machine_name is None and "machine_name" in self.model_fields_set:
            _dict['machine_name'] = None

        # set to None if machine_description (nullable) is None
        # and model_fields_set contains the field
        if self.machine_description is None and "machine_description" in self.model_fields_set:
            _dict['machine_description'] = None

        # set to None if provider_tenant_id (nullable) is None
        # and model_fields_set contains the field
        if self.provider_tenant_id is None and "provider_tenant_id" in self.model_fields_set:
            _dict['provider_tenant_id'] = None

        # set to None if tenant_id (nullable) is None
        # and model_fields_set contains the field
        if self.tenant_id is None and "tenant_id" in self.model_fields_set:
            _dict['tenant_id'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of User from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "id": obj.get("id"),
            "created": obj.get("created"),
            "provider_name": obj.get("provider_name"),
            "provider_user_id": obj.get("provider_user_id"),
            "user_type": obj.get("user_type"),
            "personal_access_tokens": [PersonalAccessToken.from_dict(_item) for _item in obj["personal_access_tokens"]] if obj.get("personal_access_tokens") is not None else None,
            "provider_user_state": obj.get("provider_user_state"),
            "user_name": obj.get("user_name"),
            "display_name": obj.get("display_name"),
            "first_name": obj.get("first_name"),
            "last_name": obj.get("last_name"),
            "email": obj.get("email"),
            "machine_name": obj.get("machine_name"),
            "machine_description": obj.get("machine_description"),
            "control_tags": [ControlTag.from_dict(_item) for _item in obj["control_tags"]] if obj.get("control_tags") is not None else None,
            "provider_tenant_id": obj.get("provider_tenant_id"),
            "tenant_id": obj.get("tenant_id")
        })
        return _obj
